@@ -20,7 +20,7 @@ public class BeanDeserializer
 {
     /* TODOs for future versions:
      * 
-     * For 2.8?
+     * For 2.9?
      *
      * - New method in JsonDeserializer (deserializeNext()) to allow use of more
      *   efficient 'nextXxx()' method `JsonParser` provides.
@@ -35,10 +35,18 @@ public class BeanDeserializer
      * Lazily constructed exception used as root cause if reporting problem
      * with creator method that returns <code>null</code> (which is not allowed)
      *
-     * @since 3.8
+     * @since 2.8
      */
     protected transient Exception _nullFromCreator;
-    
+
+    /**
+     * State marker we need in order to avoid infinite recursion for some cases
+     * (not very clean, alas, but has to do for now)
+     *
+     * @since 2.9
+     */
+    private volatile transient NameTransformer _currentlyTransforming;
+
     /*
     /**********************************************************
     /* Life-cycle, construction, initialization
@@ -86,19 +94,22 @@ public class BeanDeserializer
     }
 
     @Override
-    public JsonDeserializer<Object> unwrappingDeserializer(NameTransformer unwrapper)
+    public JsonDeserializer<Object> unwrappingDeserializer(NameTransformer transformer)
     {
-        /* bit kludgy but we don't want to accidentally change type; sub-classes
-         * MUST override this method to support unwrapped properties...
-         */
+        // bit kludgy but we don't want to accidentally change type; sub-classes
+        // MUST override this method to support unwrapped properties...
         if (getClass() != BeanDeserializer.class) {
             return this;
         }
-        /* main thing really is to just enforce ignoring of unknown
-         * properties; since there may be multiple unwrapped values
-         * and properties for all may be interleaved...
-         */
-        return new BeanDeserializer(this, unwrapper);
+        // 25-Mar-2017, tatu: Not clean at all, but for [databind#383] we do need
+        //   to keep track of accidental recursion...
+        if (_currentlyTransforming == transformer) {
+            return this;
+        }
+        _currentlyTransforming = transformer;
+        try {
+            return new BeanDeserializer(this, transformer);
+        } finally { _currentlyTransforming = null; }
     }
 
     @Override
@@ -154,34 +165,35 @@ public class BeanDeserializer
             JsonToken t) throws IOException
     {
         // and then others, generally requiring use of @JsonCreator
-        switch (t) {
-        case VALUE_STRING:
-            return deserializeFromString(p, ctxt);
-        case VALUE_NUMBER_INT:
-            return deserializeFromNumber(p, ctxt);
-        case VALUE_NUMBER_FLOAT:
-	    return deserializeFromDouble(p, ctxt);
-        case VALUE_EMBEDDED_OBJECT:
-            return deserializeFromEmbedded(p, ctxt);
-        case VALUE_TRUE:
-        case VALUE_FALSE:
-            return deserializeFromBoolean(p, ctxt);
-
-        case VALUE_NULL:
-            return deserializeFromNull(p, ctxt);
-        case START_ARRAY:
-            // these only work if there's a (delegating) creator...
-            return deserializeFromArray(p, ctxt);
-        case FIELD_NAME:
-        case END_OBJECT: // added to resolve [JACKSON-319], possible related issues
-            if (_vanillaProcessing) {
-                return vanillaDeserialize(p, ctxt, t);
+        if (t != null) {
+            switch (t) {
+            case VALUE_STRING:
+                return deserializeFromString(p, ctxt);
+            case VALUE_NUMBER_INT:
+                return deserializeFromNumber(p, ctxt);
+            case VALUE_NUMBER_FLOAT:
+                return deserializeFromDouble(p, ctxt);
+            case VALUE_EMBEDDED_OBJECT:
+                return deserializeFromEmbedded(p, ctxt);
+            case VALUE_TRUE:
+            case VALUE_FALSE:
+                return deserializeFromBoolean(p, ctxt);
+            case VALUE_NULL:
+                return deserializeFromNull(p, ctxt);
+            case START_ARRAY:
+                // these only work if there's a (delegating) creator...
+                return deserializeFromArray(p, ctxt);
+            case FIELD_NAME:
+            case END_OBJECT: // added to resolve [JACKSON-319], possible related issues
+                if (_vanillaProcessing) {
+                    return vanillaDeserialize(p, ctxt, t);
+                }
+                if (_objectIdReader != null) {
+                    return deserializeWithObjectId(p, ctxt);
+                }
+                return deserializeFromObject(p, ctxt);
+            default:
             }
-            if (_objectIdReader != null) {
-                return deserializeWithObjectId(p, ctxt);
-            }
-            return deserializeFromObject(p, ctxt);
-        default:
         }
         return ctxt.handleUnexpectedToken(handledType(), p);
     }
@@ -291,14 +303,14 @@ public class BeanDeserializer
     @Override
     public Object deserializeFromObject(JsonParser p, DeserializationContext ctxt) throws IOException
     {
-        /* 09-Dec-2014, tatu: As per [#622], we need to allow Object Id references
+        /* 09-Dec-2014, tatu: As per [databind#622], we need to allow Object Id references
          *   to come in as JSON Objects as well; but for now assume they will
          *   be simple, single-property references, which means that we can
          *   recognize them without having to buffer anything.
          *   Once again, if we must, we can do more complex handling with buffering,
          *   but let's only do that if and when that becomes necessary.
          */
-        if (_objectIdReader != null && _objectIdReader.maySerializeAsObject()) {
+        if ((_objectIdReader != null) && _objectIdReader.maySerializeAsObject()) {
             if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)
                     && _objectIdReader.isValidReferencePropertyName(p.getCurrentName(), p)) {
                 return deserializeFromObjectId(p, ctxt);
@@ -381,20 +393,29 @@ public class BeanDeserializer
     {
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(p, ctxt, _objectIdReader);
-
         TokenBuffer unknown = null;
+        final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
 
         JsonToken t = p.getCurrentToken();
         List<BeanReferring> referrings = null;
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
             String propName = p.getCurrentName();
             p.nextToken(); // to point to value
+            // Object Id property?
+            if (buffer.readIdProperty(propName)) {
+                continue;
+            }
             // creator property?
             SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
             if (creatorProp != null) {
                 // Last creator property to set?
-                if (buffer.assignParameter(creatorProp,
-                        _deserializeWithErrorWrapping(p, ctxt, creatorProp))) {
+                Object value;
+                if ((activeView != null) && !creatorProp.visibleInView(activeView)) {
+                    p.skipChildren();
+                    continue;
+                }
+                value = _deserializeWithErrorWrapping(p, ctxt, creatorProp);
+                if (buffer.assignParameter(creatorProp, value)) {
                     p.nextToken(); // to move to following FIELD_NAME/END_OBJECT
                     Object bean;
                     try {
@@ -421,10 +442,6 @@ public class BeanDeserializer
                 }
                 continue;
             }
-            // Object Id property?
-            if (buffer.readIdProperty(propName)) {
-                continue;
-            }
             // regular property? needs buffering
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) {
@@ -616,7 +633,7 @@ public class BeanDeserializer
             p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) { // normal case
-                if (activeView != null && !prop.visibleInView(activeView)) {
+                if ((activeView != null) && !prop.visibleInView(activeView)) {
                     p.skipChildren();
                     continue;
                 }
@@ -640,21 +657,17 @@ public class BeanDeserializer
                 // but... others should be passed to unwrapped property deserializers
                 tokens.writeFieldName(propName);
                 tokens.copyCurrentStructure(p);
-            } else {
-                // Need to copy to a separate buffer first
-                TokenBuffer b2 = new TokenBuffer(p, ctxt);
-                b2.copyCurrentStructure(p);
-                tokens.writeFieldName(propName);
-                tokens.append(b2);
-                try {
-                    JsonParser p2 = b2.asParser(p);
-                    p2.nextToken();
-                    _anySetter.deserializeAndSet(p2, ctxt, bean, propName);
-                } catch (Exception e) {
-                    wrapAndThrow(e, bean, propName, ctxt);
-                }
                 continue;
             }
+            // Need to copy to a separate buffer first
+            TokenBuffer b2 = TokenBuffer.asCopyOfValue(p);
+            tokens.writeFieldName(propName);
+            tokens.append(b2);
+            try {
+                _anySetter.deserializeAndSet(b2.asParserOnFirstToken(), ctxt, bean, propName);
+            } catch (Exception e) {
+                wrapAndThrow(e, bean, propName, ctxt);
+            }
         }
         tokens.writeEndObject();
         _unwrappedPropertyHandler.processUnwrapped(p, ctxt, bean, tokens);
@@ -662,7 +675,8 @@ public class BeanDeserializer
     }
 
     @SuppressWarnings("resource")
-    protected Object deserializeWithUnwrapped(JsonParser p, DeserializationContext ctxt, Object bean)
+    protected Object deserializeWithUnwrapped(JsonParser p, DeserializationContext ctxt,
+            Object bean)
         throws IOException
     {
         JsonToken t = p.getCurrentToken();
@@ -702,14 +716,11 @@ public class BeanDeserializer
                 tokens.copyCurrentStructure(p);
             } else {
                 // Need to copy to a separate buffer first
-                TokenBuffer b2 = new TokenBuffer(p, ctxt);
-                b2.copyCurrentStructure(p);
+                TokenBuffer b2 = TokenBuffer.asCopyOfValue(p);
                 tokens.writeFieldName(propName);
                 tokens.append(b2);
                 try {
-                    JsonParser p2 = b2.asParser(p);
-                    p2.nextToken();
-                    _anySetter.deserializeAndSet(p2, ctxt, bean, propName);
+                    _anySetter.deserializeAndSet(b2.asParserOnFirstToken(), ctxt, bean, propName);
                 } catch (Exception e) {
                     wrapAndThrow(e, bean, propName, ctxt);
                 }
@@ -725,6 +736,10 @@ public class BeanDeserializer
     protected Object deserializeUsingPropertyBasedWithUnwrapped(JsonParser p, DeserializationContext ctxt)
         throws IOException
     {
+        // 01-Dec-2016, tatu: Note: This IS legal to call, but only when unwrapped
+        //    value itself is NOT passed via `CreatorProperty` (which isn't supported).
+        //    Ok however to pass via setter or field.
+        
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(p, ctxt, _objectIdReader);
 
@@ -739,7 +754,8 @@ public class BeanDeserializer
             SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
             if (creatorProp != null) {
                 // Last creator property to set?
-                if (buffer.assignParameter(creatorProp, _deserializeWithErrorWrapping(p, ctxt, creatorProp))) {
+                if (buffer.assignParameter(creatorProp,
+                        _deserializeWithErrorWrapping(p, ctxt, creatorProp))) {
                     t = p.nextToken(); // to move to following FIELD_NAME/END_OBJECT
                     Object bean;
                     try {
@@ -751,16 +767,23 @@ public class BeanDeserializer
                     p.setCurrentValue(bean);
                     // if so, need to copy all remaining tokens into buffer
                     while (t == JsonToken.FIELD_NAME) {
-                        p.nextToken(); // to skip name
+                        // NOTE: do NOT skip name as it needs to be copied; `copyCurrentStructure` does that
                         tokens.copyCurrentStructure(p);
                         t = p.nextToken();
                     }
+                    // 28-Aug-2018, tatu: Let's add sanity check here, easier to catch off-by-some
+                    //    problems if we maintain invariants
+                    if (t != JsonToken.END_OBJECT) {
+                        ctxt.reportWrongTokenException(this, JsonToken.END_OBJECT, 
+                                "Attempted to unwrap '%s' value",
+                                handledType().getName());
+                    }
                     tokens.writeEndObject();
                     if (bean.getClass() != _beanType.getRawClass()) {
                         // !!! 08-Jul-2011, tatu: Could probably support; but for now
                         //   it's too complicated, so bail out
-                        tokens.close();
-                        ctxt.reportMappingException("Can not create polymorphic instances with unwrapped values");
+                        ctxt.reportInputMismatch(creatorProp,
+                                "Cannot create polymorphic instances with unwrapped values");
                         return null;
                     }
                     return _unwrappedPropertyHandler.processUnwrapped(p, ctxt, bean, tokens);
@@ -792,15 +815,12 @@ public class BeanDeserializer
                 tokens.copyCurrentStructure(p);
             } else {
                 // Need to copy to a separate buffer first
-                TokenBuffer b2 = new TokenBuffer(p, ctxt);
-                b2.copyCurrentStructure(p);
+                TokenBuffer b2 = TokenBuffer.asCopyOfValue(p);
                 tokens.writeFieldName(propName);
                 tokens.append(b2);
                 try {
-                    JsonParser p2 = b2.asParser(p);
-                    p2.nextToken();
                     buffer.bufferAnyProperty(_anySetter, propName,
-                            _anySetter.deserialize(p2, ctxt));
+                            _anySetter.deserialize(b2.asParserOnFirstToken(), ctxt));
                 } catch (Exception e) {
                     wrapAndThrow(e, _beanType.getRawClass(), propName, ctxt);
                 }
@@ -940,8 +960,9 @@ public class BeanDeserializer
                         if (bean.getClass() != _beanType.getRawClass()) {
                             // !!! 08-Jul-2011, tatu: Could theoretically support; but for now
                             //   it's too complicated, so bail out
-                            ctxt.reportMappingException("Can not create polymorphic instances with external type ids");
-                            return null;
+                            return ctxt.reportBadDefinition(_beanType, String.format(
+                                    "Cannot create polymorphic instances with external type ids (%s -> %s)",
+                                    _beanType, bean.getClass()));
                         }
                         return ext.complete(p, ctxt, bean);
                     }
@@ -969,9 +990,11 @@ public class BeanDeserializer
             }
             // "any property"?
             if (_anySetter != null) {
-                buffer.bufferAnyProperty(_anySetter, propName, _anySetter.deserialize(p, ctxt));
+                buffer.bufferAnyProperty(_anySetter, propName,
+                        _anySetter.deserialize(p, ctxt));
             }
         }
+        tokens.writeEndObject();
 
         // We hit END_OBJECT; resolve the pieces:
         try {
@@ -1019,8 +1042,8 @@ public class BeanDeserializer
         public void handleResolvedForwardReference(Object id, Object value) throws IOException
         {
             if (_bean == null) {
-                _context.reportMappingException(
-"Can not resolve ObjectId forward reference using property '%s' (of type %s): Bean not yet resolved",
+                _context.reportInputMismatch(_prop,
+"Cannot resolve ObjectId forward reference using property '%s' (of type %s): Bean not yet resolved",
 _prop.getName(), _prop.getDeclaringClass().getName());
         }
             _prop.set(_bean, value);
