@@ -26,7 +26,15 @@ public class BuilderBasedDeserializer
     private static final long serialVersionUID = 1L;
 
     protected final AnnotatedMethod _buildMethod;
-	
+
+    /**
+     * Type that the builder will produce, target type; as opposed to
+     * `handledType()` which refers to Builder class.
+     *
+     * @since 2.9
+     */
+    protected final JavaType _targetType;
+
     /*
     /**********************************************************
     /* Life-cycle, construction, initialization
@@ -37,21 +45,37 @@ public class BuilderBasedDeserializer
      * Constructor used by {@link BeanDeserializerBuilder}.
      */
     public BuilderBasedDeserializer(BeanDeserializerBuilder builder,
-            BeanDescription beanDesc,
+            BeanDescription beanDesc, JavaType targetType,
             BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs,
             Set<String> ignorableProps, boolean ignoreAllUnknown,
             boolean hasViews)
     {
         super(builder, beanDesc, properties, backRefs,
                 ignorableProps, ignoreAllUnknown, hasViews);
+        _targetType = targetType;
         _buildMethod = builder.getBuildMethod();
-        // 05-Mar-2012, tatu: Can not really make Object Ids work with builders, not yet anyway
+        // 05-Mar-2012, tatu: Cannot really make Object Ids work with builders, not yet anyway
         if (_objectIdReader != null) {
-            throw new IllegalArgumentException("Can not use Object Id with Builder-based deserialization (type "
+            throw new IllegalArgumentException("Cannot use Object Id with Builder-based deserialization (type "
                     +beanDesc.getType()+")");
         }
     }
 
+    /**
+     * @deprecated Since 2.9
+     */
+    @Deprecated
+    public BuilderBasedDeserializer(BeanDeserializerBuilder builder,
+            BeanDescription beanDesc,
+            BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs,
+            Set<String> ignorableProps, boolean ignoreAllUnknown,
+            boolean hasViews)
+    {
+        this(builder, beanDesc,
+                beanDesc.getType(), // Wrong! But got no access via `BeanDeserializerBuilder`
+                properties, backRefs, ignorableProps, ignoreAllUnknown, hasViews);
+    }
+
     /**
      * Copy-constructor that can be used by sub-classes to allow
      * copy-on-write styling copying of settings of an existing instance.
@@ -65,28 +89,33 @@ public class BuilderBasedDeserializer
     {
         super(src, ignoreAllUnknown);
         _buildMethod = src._buildMethod;
+        _targetType = src._targetType;
     }
-    
+
     protected BuilderBasedDeserializer(BuilderBasedDeserializer src, NameTransformer unwrapper) {
         super(src, unwrapper);
         _buildMethod = src._buildMethod;
+        _targetType = src._targetType;
     }
 
     public BuilderBasedDeserializer(BuilderBasedDeserializer src, ObjectIdReader oir) {
         super(src, oir);
         _buildMethod = src._buildMethod;
+        _targetType = src._targetType;
     }
 
     public BuilderBasedDeserializer(BuilderBasedDeserializer src, Set<String> ignorableProps) {
         super(src, ignorableProps);
         _buildMethod = src._buildMethod;
+        _targetType = src._targetType;
     }
 
     public BuilderBasedDeserializer(BuilderBasedDeserializer src, BeanPropertyMap props) {
         super(src, props);
         _buildMethod = src._buildMethod;
+        _targetType = src._targetType;
     }
-    
+
     @Override
     public JsonDeserializer<Object> unwrappingDeserializer(NameTransformer unwrapper)
     {
@@ -115,7 +144,19 @@ public class BuilderBasedDeserializer
     @Override
     protected BeanDeserializerBase asArrayDeserializer() {
         SettableBeanProperty[] props = _beanProperties.getPropertiesInInsertionOrder();
-        return new BeanAsArrayBuilderDeserializer(this, props, _buildMethod);
+        return new BeanAsArrayBuilderDeserializer(this, _targetType, props, _buildMethod);
+    }
+
+    /*
+    /**********************************************************
+    /* JsonDeserializer implementation
+    /**********************************************************
+     */
+
+    @Override // since 2.9
+    public Boolean supportsUpdate(DeserializationConfig config) {
+        // 26-Oct-2016, tatu: No, we can't merge Builder-based POJOs as of now
+        return Boolean.FALSE;
     }
     
     /*
@@ -123,8 +164,8 @@ public class BuilderBasedDeserializer
     /* JsonDeserializer implementation
     /**********************************************************
      */
-    
-    protected final Object finishBuild(DeserializationContext ctxt, Object builder)
+
+    protected Object finishBuild(DeserializationContext ctxt, Object builder)
             throws IOException
     {
         // As per [databind#777], allow returning builder itself
@@ -132,52 +173,48 @@ public class BuilderBasedDeserializer
             return builder;
         }
         try {
-            return _buildMethod.getMember().invoke(builder);
+            return _buildMethod.getMember().invoke(builder, (Object[]) null);
         } catch (Exception e) {
             return wrapInstantiationProblem(e, ctxt);
         }
     }
-    
+
     /**
      * Main deserialization method for bean-based objects (POJOs).
      */
     @Override
-    public final Object deserialize(JsonParser p, DeserializationContext ctxt)
+    public Object deserialize(JsonParser p, DeserializationContext ctxt)
         throws IOException
     {
-        JsonToken t = p.getCurrentToken();
-        
         // common case first:
-        if (t == JsonToken.START_OBJECT) {
-            t = p.nextToken();
+        if (p.isExpectedStartObjectToken()) {
+            JsonToken t = p.nextToken();
             if (_vanillaProcessing) {
-            	return finishBuild(ctxt, vanillaDeserialize(p, ctxt, t));
+                return finishBuild(ctxt, vanillaDeserialize(p, ctxt, t));
             }
             Object builder = deserializeFromObject(p, ctxt);
             return finishBuild(ctxt, builder);
         }
         // and then others, generally requiring use of @JsonCreator
-        if (t != null) {
-            switch (t) {
-            case VALUE_STRING:
-                return finishBuild(ctxt, deserializeFromString(p, ctxt));
-            case VALUE_NUMBER_INT:
-                return finishBuild(ctxt, deserializeFromNumber(p, ctxt));
-            case VALUE_NUMBER_FLOAT:
-            	return finishBuild(ctxt, deserializeFromDouble(p, ctxt));
-            case VALUE_EMBEDDED_OBJECT:
-                return p.getEmbeddedObject();
-            case VALUE_TRUE:
-            case VALUE_FALSE:
-                return finishBuild(ctxt, deserializeFromBoolean(p, ctxt));
-            case START_ARRAY:
-                // these only work if there's a (delegating) creator...
-                return finishBuild(ctxt, deserializeFromArray(p, ctxt));
-            case FIELD_NAME:
-            case END_OBJECT:
-                return finishBuild(ctxt, deserializeFromObject(p, ctxt));
-            default:
-            }
+        switch (p.getCurrentTokenId()) {
+        case JsonTokenId.ID_STRING:
+            return finishBuild(ctxt, deserializeFromString(p, ctxt));
+        case JsonTokenId.ID_NUMBER_INT:
+            return finishBuild(ctxt, deserializeFromNumber(p, ctxt));
+        case JsonTokenId.ID_NUMBER_FLOAT:
+            return finishBuild(ctxt, deserializeFromDouble(p, ctxt));
+        case JsonTokenId.ID_EMBEDDED_OBJECT:
+            return p.getEmbeddedObject();
+        case JsonTokenId.ID_TRUE:
+        case JsonTokenId.ID_FALSE:
+            return finishBuild(ctxt, deserializeFromBoolean(p, ctxt));
+        case JsonTokenId.ID_START_ARRAY:
+            // these only work if there's a (delegating) creator...
+            return finishBuild(ctxt, deserializeFromArray(p, ctxt));
+        case JsonTokenId.ID_FIELD_NAME:
+        case JsonTokenId.ID_END_OBJECT:
+            return finishBuild(ctxt, deserializeFromObject(p, ctxt));
+        default:
         }
         return ctxt.handleUnexpectedToken(handledType(), p);
     }
@@ -189,74 +226,40 @@ public class BuilderBasedDeserializer
      */
     @Override
     public Object deserialize(JsonParser p, DeserializationContext ctxt,
-    		Object builder)
-        throws IOException
+    		Object value) throws IOException
     {
-        /* Important: we call separate method which does NOT call
-         * 'finishBuild()', to avoid problems with recursion
-         */
-        return finishBuild(ctxt, _deserialize(p, ctxt, builder));
+        // 26-Oct-2016, tatu: I cannot see any of making this actually
+        //    work correctly, so let's indicate problem right away
+        JavaType valueType = _targetType;
+        // Did they try to give us builder?
+        Class<?> builderRawType = handledType();
+        Class<?> instRawType = value.getClass();
+        if (builderRawType.isAssignableFrom(instRawType)) {
+            return ctxt.reportBadDefinition(valueType, String.format(
+                    "Deserialization of %s by passing existing Builder (%s) instance not supported",
+                    valueType, builderRawType.getName()));
+        }
+        return ctxt.reportBadDefinition(valueType, String.format(
+                "Deserialization of %s by passing existing instance (of %s) not supported",
+                valueType, instRawType.getName()));
     }
-    
+
     /*
     /**********************************************************
     /* Concrete deserialization methods
     /**********************************************************
      */
 
-    protected final Object _deserialize(JsonParser p,
-            DeserializationContext ctxt, Object builder)
-        throws IOException, JsonProcessingException
-    {        
-        if (_injectables != null) {
-            injectValues(ctxt, builder);
-        }
-        if (_unwrappedPropertyHandler != null) {
-            return deserializeWithUnwrapped(p, ctxt, builder);
-        }
-        if (_externalTypeIdHandler != null) {
-            return deserializeWithExternalTypeId(p, ctxt, builder);
-        }
-        if (_needViewProcesing) {
-            Class<?> view = ctxt.getActiveView();
-            if (view != null) {
-                return deserializeWithView(p, ctxt, builder, view);
-            }
-        }
-        JsonToken t = p.getCurrentToken();
-        // 23-Mar-2010, tatu: In some cases, we start with full JSON object too...
-        if (t == JsonToken.START_OBJECT) {
-            t = p.nextToken();
-        }
-        for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
-            // Skip field name:
-            p.nextToken();
-            SettableBeanProperty prop = _beanProperties.find(propName);
-            
-            if (prop != null) { // normal case
-                try {
-                    builder = prop.deserializeSetAndReturn(p, ctxt, builder);
-                } catch (Exception e) {
-                    wrapAndThrow(e, builder, propName, ctxt);
-                }
-                continue;
-            }
-            handleUnknownVanilla(p, ctxt, handledType(), propName);
-        }
-        return builder;
-    }
-    
     /**
      * Streamlined version that is only used when no "special"
      * features are enabled.
      */
     private final Object vanillaDeserialize(JsonParser p,
     		DeserializationContext ctxt, JsonToken t)
-        throws IOException, JsonProcessingException
+        throws IOException
     {
         Object bean = _valueInstantiator.createUsingDefault(ctxt);
-        for (; p.getCurrentToken() != JsonToken.END_OBJECT; p.nextToken()) {
+        for (; p.getCurrentToken() == JsonToken.FIELD_NAME; p.nextToken()) {
             String propName = p.getCurrentName();
             // Skip field name:
             p.nextToken();
@@ -280,7 +283,7 @@ public class BuilderBasedDeserializer
      */
     @Override
     public Object deserializeFromObject(JsonParser p, DeserializationContext ctxt)
-        throws IOException, JsonProcessingException
+        throws IOException
     {
         if (_nonStandardCreation) {
             if (_unwrappedPropertyHandler != null) {
@@ -301,7 +304,7 @@ public class BuilderBasedDeserializer
                 return deserializeWithView(p, ctxt, bean, view);
             }
         }
-        for (; p.getCurrentToken() != JsonToken.END_OBJECT; p.nextToken()) {
+        for (; p.getCurrentToken() == JsonToken.FIELD_NAME; p.nextToken()) {
             String propName = p.getCurrentName();
             // Skip field name:
             p.nextToken();
@@ -323,18 +326,21 @@ public class BuilderBasedDeserializer
      * Method called to deserialize bean using "property-based creator":
      * this means that a non-default constructor or factory method is
      * called, and then possibly other setters. The trick is that
-     * values for creator method need to be buffered, first; and 
+     * values for creator method need to be buffered, first; and
      * due to non-guaranteed ordering possibly some other properties
      * as well.
+     *
+     * @return Builder instance constructed
      */
     @Override
     @SuppressWarnings("resource")
-    protected final Object _deserializeUsingPropertyBased(final JsonParser p,
+    protected Object _deserializeUsingPropertyBased(final JsonParser p,
             final DeserializationContext ctxt)
-        throws IOException, JsonProcessingException
+        throws IOException
     { 
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(p, ctxt, _objectIdReader);
+        final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
 
         // 04-Jan-2010, tatu: May need to collect unknown properties for polymorphic cases
         TokenBuffer unknown = null;
@@ -346,25 +352,29 @@ public class BuilderBasedDeserializer
             // creator property?
             SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
             if (creatorProp != null) {
+                if ((activeView != null) && !creatorProp.visibleInView(activeView)) {
+                    p.skipChildren();
+                    continue;
+                }
                 // Last creator property to set?
                 if (buffer.assignParameter(creatorProp, creatorProp.deserialize(p, ctxt))) {
                     p.nextToken(); // to move to following FIELD_NAME/END_OBJECT
-                    Object bean;
+                    Object builder;
                     try {
-                        bean = creator.build(ctxt, buffer);
+                        builder = creator.build(ctxt, buffer);
                     } catch (Exception e) {
                         wrapAndThrow(e, _beanType.getRawClass(), propName, ctxt);
                         continue; // never gets here
                     }
                     //  polymorphic?
-                    if (bean.getClass() != _beanType.getRawClass()) {
-                        return handlePolymorphic(p, ctxt, bean, unknown);
+                    if (builder.getClass() != _beanType.getRawClass()) {
+                        return handlePolymorphic(p, ctxt, builder, unknown);
                     }
                     if (unknown != null) { // nope, just extra unknown stuff...
-                        bean = handleUnknownProperties(ctxt, bean, unknown);
+                        builder = handleUnknownProperties(ctxt, builder, unknown);
                     }
                     // or just clean?
-                    return _deserialize(p, ctxt, bean);
+                    return _deserialize(p, ctxt, builder);
                 }
                 continue;
             }
@@ -398,32 +408,80 @@ public class BuilderBasedDeserializer
         }
 
         // We hit END_OBJECT, so:
-        Object bean;
+        Object builder;
         try {
-            bean = creator.build(ctxt, buffer);
+            builder = creator.build(ctxt, buffer);
         } catch (Exception e) {
-            bean = wrapInstantiationProblem(e, ctxt);
+            builder = wrapInstantiationProblem(e, ctxt);
         }
         if (unknown != null) {
             // polymorphic?
-            if (bean.getClass() != _beanType.getRawClass()) {
-                return handlePolymorphic(null, ctxt, bean, unknown);
+            if (builder.getClass() != _beanType.getRawClass()) {
+                return handlePolymorphic(null, ctxt, builder, unknown);
             }
             // no, just some extra unknown properties
-            return handleUnknownProperties(ctxt, bean, unknown);
+            return handleUnknownProperties(ctxt, builder, unknown);
         }
-        return bean;
+        return builder;
     }
-    
+
+    @SuppressWarnings("resource")
+    protected final Object _deserialize(JsonParser p,
+            DeserializationContext ctxt, Object builder) throws IOException
+    {        
+        if (_injectables != null) {
+            injectValues(ctxt, builder);
+        }
+        if (_unwrappedPropertyHandler != null) {
+            if (p.hasToken(JsonToken.START_OBJECT)) {
+                p.nextToken();
+            }
+            TokenBuffer tokens = new TokenBuffer(p, ctxt);
+            tokens.writeStartObject();
+            return deserializeWithUnwrapped(p, ctxt, builder, tokens);
+        }
+        if (_externalTypeIdHandler != null) {
+            return deserializeWithExternalTypeId(p, ctxt, builder);
+        }
+        if (_needViewProcesing) {
+            Class<?> view = ctxt.getActiveView();
+            if (view != null) {
+                return deserializeWithView(p, ctxt, builder, view);
+            }
+        }
+        JsonToken t = p.getCurrentToken();
+        // 23-Mar-2010, tatu: In some cases, we start with full JSON object too...
+        if (t == JsonToken.START_OBJECT) {
+            t = p.nextToken();
+        }
+        for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
+            String propName = p.getCurrentName();
+            // Skip field name:
+            p.nextToken();
+            SettableBeanProperty prop = _beanProperties.find(propName);
+            
+            if (prop != null) { // normal case
+                try {
+                    builder = prop.deserializeSetAndReturn(p, ctxt, builder);
+                } catch (Exception e) {
+                    wrapAndThrow(e, builder, propName, ctxt);
+                }
+                continue;
+            }
+            handleUnknownVanilla(p, ctxt, handledType(), propName);
+        }
+        return builder;
+    }
+
     /*
     /**********************************************************
     /* Deserializing when we have to consider an active View
     /**********************************************************
      */
-    
+
     protected final Object deserializeWithView(JsonParser p, DeserializationContext ctxt,
             Object bean, Class<?> activeView)
-        throws IOException, JsonProcessingException
+        throws IOException
     {
         JsonToken t = p.getCurrentToken();
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
@@ -447,7 +505,7 @@ public class BuilderBasedDeserializer
         }
         return bean;
     }
-    
+
     /*
     /**********************************************************
     /* Handling for cases where we have "unwrapped" values
@@ -460,7 +518,7 @@ public class BuilderBasedDeserializer
      */
     @SuppressWarnings("resource")
     protected Object deserializeWithUnwrapped(JsonParser p, DeserializationContext ctxt)
-        throws IOException, JsonProcessingException
+        throws IOException
     {
         if (_delegateDeserializer != null) {
             return _valueInstantiator.createUsingDelegate(ctxt, _delegateDeserializer.deserialize(p, ctxt));
@@ -477,8 +535,7 @@ public class BuilderBasedDeserializer
         }
 
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
-        
-        for (; p.getCurrentToken() != JsonToken.END_OBJECT; p.nextToken()) {
+        for (; p.getCurrentToken() == JsonToken.FIELD_NAME; p.nextToken()) {
             String propName = p.getCurrentName();
             p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
@@ -513,65 +570,20 @@ public class BuilderBasedDeserializer
             }
         }
         tokens.writeEndObject();
-        _unwrappedPropertyHandler.processUnwrapped(p, ctxt, bean, tokens);
-        return bean;
-    }    
-
-    @SuppressWarnings("resource")
-    protected Object deserializeWithUnwrapped(JsonParser p,
-    		DeserializationContext ctxt, Object bean)
-        throws IOException, JsonProcessingException
-    {
-        JsonToken t = p.getCurrentToken();
-        if (t == JsonToken.START_OBJECT) {
-            t = p.nextToken();
-        }
-        TokenBuffer tokens = new TokenBuffer(p, ctxt);
-        tokens.writeStartObject();
-        final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
-        for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
-            SettableBeanProperty prop = _beanProperties.find(propName);
-            p.nextToken();
-            if (prop != null) { // normal case
-                if (activeView != null && !prop.visibleInView(activeView)) {
-                    p.skipChildren();
-                    continue;
-                }
-                try {
-                    bean = prop.deserializeSetAndReturn(p, ctxt, bean);
-                } catch (Exception e) {
-                    wrapAndThrow(e, bean, propName, ctxt);
-                }
-                continue;
-            }
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                handleIgnoredProperty(p, ctxt, bean, propName);
-                continue;
-            }
-            // but... others should be passed to unwrapped property deserializers
-            tokens.writeFieldName(propName);
-            tokens.copyCurrentStructure(p);
-            // how about any setter? We'll get copies but...
-            if (_anySetter != null) {
-                _anySetter.deserializeAndSet(p, ctxt, bean, propName);
-            }
-        }
-        tokens.writeEndObject();
-        _unwrappedPropertyHandler.processUnwrapped(p, ctxt, bean, tokens);
-        return bean;
+        return _unwrappedPropertyHandler.processUnwrapped(p, ctxt, bean, tokens);
     }
 
     @SuppressWarnings("resource")
     protected Object deserializeUsingPropertyBasedWithUnwrapped(JsonParser p,
     		DeserializationContext ctxt)
-        throws IOException, JsonProcessingException
+        throws IOException
     {
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(p, ctxt, _objectIdReader);
 
         TokenBuffer tokens = new TokenBuffer(p, ctxt);
         tokens.writeStartObject();
+        Object builder = null;
 
         JsonToken t = p.getCurrentToken();
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
@@ -583,27 +595,16 @@ public class BuilderBasedDeserializer
                 // Last creator property to set?
                 if (buffer.assignParameter(creatorProp, creatorProp.deserialize(p, ctxt))) {
                     t = p.nextToken(); // to move to following FIELD_NAME/END_OBJECT
-                    Object bean;
                     try {
-                        bean = creator.build(ctxt, buffer);
+                        builder = creator.build(ctxt, buffer);
                     } catch (Exception e) {
                         wrapAndThrow(e, _beanType.getRawClass(), propName, ctxt);
                         continue; // never gets here
                     }
-                    // if so, need to copy all remaining tokens into buffer
-                    while (t == JsonToken.FIELD_NAME) {
-                        p.nextToken(); // to skip name
-                        tokens.copyCurrentStructure(p);
-                        t = p.nextToken();
+                    if (builder.getClass() != _beanType.getRawClass()) {
+                        return handlePolymorphic(p, ctxt, builder, tokens);
                     }
-                    tokens.writeEndObject();
-                    if (bean.getClass() != _beanType.getRawClass()) {
-                        // !!! 08-Jul-2011, tatu: Could probably support; but for now
-                        //   it's too complicated, so bail out
-                        ctxt.reportMappingException("Can not create polymorphic instances with unwrapped values");
-                        return null;
-                    }
-                    return _unwrappedPropertyHandler.processUnwrapped(p, ctxt, bean, tokens);
+                    return deserializeWithUnwrapped(p, ctxt, builder, tokens);
                 }
                 continue;
             }
@@ -628,16 +629,54 @@ public class BuilderBasedDeserializer
                 buffer.bufferAnyProperty(_anySetter, propName, _anySetter.deserialize(p, ctxt));
             }
         }
+        tokens.writeEndObject();
 
         // We hit END_OBJECT, so:
-        Object bean;
-        // !!! 15-Feb-2012, tatu: Need to modify creator to use Builder!
-        try {
-            bean = creator.build(ctxt, buffer);
-        } catch (Exception e) {
-            return wrapInstantiationProblem(e, ctxt);
+        if (builder == null) {
+            try {
+                builder = creator.build(ctxt, buffer);
+            } catch (Exception e) {
+                return wrapInstantiationProblem(e, ctxt);
+            }
         }
-        return _unwrappedPropertyHandler.processUnwrapped(p, ctxt, bean, tokens);
+        return _unwrappedPropertyHandler.processUnwrapped(p, ctxt, builder, tokens);
+    }
+
+    protected Object deserializeWithUnwrapped(JsonParser p,
+            DeserializationContext ctxt, Object builder, TokenBuffer tokens)
+        throws IOException
+    {
+        final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
+        for (JsonToken t = p.getCurrentToken(); t == JsonToken.FIELD_NAME; t = p.nextToken()) {
+            String propName = p.getCurrentName();
+            SettableBeanProperty prop = _beanProperties.find(propName);
+            p.nextToken();
+            if (prop != null) { // normal case
+                if (activeView != null && !prop.visibleInView(activeView)) {
+                    p.skipChildren();
+                    continue;
+                }
+                try {
+                    builder = prop.deserializeSetAndReturn(p, ctxt, builder);
+                } catch (Exception e) {
+                    wrapAndThrow(e, builder, propName, ctxt);
+                }
+                continue;
+            }
+            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+                handleIgnoredProperty(p, ctxt, builder, propName);
+                continue;
+            }
+            // but... others should be passed to unwrapped property deserializers
+            tokens.writeFieldName(propName);
+            tokens.copyCurrentStructure(p);
+            // how about any setter? We'll get copies but...
+            if (_anySetter != null) {
+                _anySetter.deserializeAndSet(p, ctxt, builder, propName);
+            }
+        }
+        tokens.writeEndObject();
+        return _unwrappedPropertyHandler.processUnwrapped(p, ctxt, builder, tokens);
     }
 
     /*
@@ -646,9 +685,9 @@ public class BuilderBasedDeserializer
     /* external type id
     /**********************************************************
      */
-    
+
     protected Object deserializeWithExternalTypeId(JsonParser p, DeserializationContext ctxt)
-        throws IOException, JsonProcessingException
+        throws IOException
     {
         if (_propertyBasedCreator != null) {
             return deserializeUsingPropertyBasedWithExternalTypeId(p, ctxt);
@@ -658,7 +697,7 @@ public class BuilderBasedDeserializer
 
     protected Object deserializeWithExternalTypeId(JsonParser p,
     		DeserializationContext ctxt, Object bean)
-        throws IOException, JsonProcessingException
+        throws IOException
     {
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
         final ExternalTypeHandler ext = _externalTypeIdHandler.start();
@@ -668,7 +707,7 @@ public class BuilderBasedDeserializer
             t = p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) { // normal case
-                // [JACKSON-831]: may have property AND be used as external type id:
+                // May have property AND be used as external type id:
                 if (t.isScalarValue()) {
                     ext.handleTypePropertyValue(p, ctxt, propName, bean);
                 }
@@ -702,18 +741,21 @@ public class BuilderBasedDeserializer
                 continue;
             } else {
                 // Unknown: let's call handler method
-                handleUnknownProperty(p, ctxt, bean, propName);         
+                handleUnknownProperty(p, ctxt, bean, propName);
             }
         }
         // and when we get this far, let's try finalizing the deal:
         return ext.complete(p, ctxt, bean);
-    }        
+    }
 
     protected Object deserializeUsingPropertyBasedWithExternalTypeId(JsonParser p,
     		DeserializationContext ctxt)
-        throws IOException, JsonProcessingException
+        throws IOException
     {
         // !!! 04-Mar-2012, TODO: Need to fix -- will not work as is...
-        throw new IllegalStateException("Deserialization with Builder, External type id, @JsonCreator not yet implemented");
+        JavaType t = _targetType;
+        return ctxt.reportBadDefinition(t, String.format(
+                "Deserialization (of %s) with Builder, External type id, @JsonCreator not yet implemented",
+                t));
     }
 }
