@@ -13,16 +13,42 @@ import java.util.TimeZone;
 import java.util.regex.Pattern;
 
 import com.fasterxml.jackson.core.*;
-import com.fasterxml.jackson.databind.DeserializationContext;
-import com.fasterxml.jackson.databind.DeserializationFeature;
-import com.fasterxml.jackson.databind.JavaType;
-import com.fasterxml.jackson.databind.JsonMappingException;
+import com.fasterxml.jackson.core.util.VersionUtil;
+import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.exc.InvalidFormatException;
 import com.fasterxml.jackson.databind.util.ClassUtil;
 
 /**
- * Base class for simple deserializers that only accept JSON String
- * values as the source.
+ * Base class for simple deserializers that serialize values from String
+ * representation: this includes JSON Strings and other Scalar values that
+ * can be coerced into text, like Numbers and Booleans).
+ * Simple JSON String values are trimmed using {@link java.lang.String#trim}.
+ * Partial deserializer implementation will try to first access current token as
+ * a String, calls {@link #_deserialize(String,DeserializationContext)} and
+ * returns return value.
+ * If this does not work (current token not a simple scalar type), attempts
+ * are made so that:
+ *<ul>
+ * <li>Embedded values ({@link JsonToken#VALUE_EMBEDDED_OBJECT}) are returned as-is
+ *    if they are of compatible type
+ *  </li>
+ * <li>Arrays may be "unwrapped" if (and only if) {@link DeserializationFeature#UNWRAP_SINGLE_VALUE_ARRAYS}
+ *    is enabled, and array contains just a single scalar value that can be deserialized
+ *    (for example, JSON Array with single JSON String element).
+ *  </li>
+ * </ul>
+ *<p>
+ * Special handling includes:
+ * <ul>
+ * <li>Null values ({@link JsonToken#VALUE_NULL}) are handled by returning value
+ *   returned by {@link JsonDeserializer#getNullValue(DeserializationContext)}: default
+ *   implementation simply returns Java `null` but this may be overridden.
+ *  </li>
+ * <li>Empty String (after trimming) will result in {@link #_deserializeFromEmptyString}
+ *   getting called, and return value being returned as deserialization: default implementation
+ *   simply returns `null`.
+ *  </li>
+ * </ul>
  */
 @SuppressWarnings("serial")
 public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
@@ -99,53 +125,43 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
     /* Deserializer implementations
     /**********************************************************
      */
-    
+
     @SuppressWarnings("unchecked")
     @Override
     public T deserialize(JsonParser p, DeserializationContext ctxt) throws IOException
     {
-        // 22-Sep-2012, tatu: For 2.1, use this new method, may force coercion:
+        // Let's get textual value, possibly via coercion from other scalar types
         String text = p.getValueAsString();
         if (text != null) { // has String representation
             if (text.length() == 0 || (text = text.trim()).length() == 0) {
-                // 04-Feb-2013, tatu: Usually should become null; but not always
+                // Usually should become null; but not always
                 return _deserializeFromEmptyString();
             }
             Exception cause = null;
             try {
-                T result = _deserialize(text, ctxt);
-                if (result != null) {
-                    return result;
-                }
-            } catch (IllegalArgumentException iae) {
-                cause = iae;
-            } catch (MalformedURLException me) {
-                cause = me;
+                // 19-May-2017, tatu: Used to require non-null result (assuming `null`
+                //    indicated error; but that seems wrong. Should be able to return
+                //    `null` as value.
+                return _deserialize(text, ctxt);
+            } catch (IllegalArgumentException | MalformedURLException e) {
+                cause = e;
             }
+            // note: `cause` can't be null
             String msg = "not a valid textual representation";
-            if (cause != null) {
-                String m2 = cause.getMessage();
-                if (m2 != null) {
-                    msg = msg + ", problem: "+m2;
-                }
+            String m2 = cause.getMessage();
+            if (m2 != null) {
+                msg = msg + ", problem: "+m2;
             }
             // 05-May-2016, tatu: Unlike most usage, this seems legit, so...
             JsonMappingException e = ctxt.weirdStringException(text, _valueClass, msg);
-            if (cause != null) {
-                e.initCause(cause);
-            }
+            e.initCause(cause);
             throw e;
             // nothing to do here, yet? We'll fail anyway
         }
         JsonToken t = p.getCurrentToken();
         // [databind#381]
-        if (t == JsonToken.START_ARRAY && ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
-            p.nextToken();
-            final T value = deserialize(p, ctxt);
-            if (p.nextToken() != JsonToken.END_ARRAY) {
-                handleMissingEndArrayForSingle(p, ctxt);
-            }
-            return value;
+        if (t == JsonToken.START_ARRAY) {
+            return _deserializeFromArray(p, ctxt);
         }
         if (t == JsonToken.VALUE_EMBEDDED_OBJECT) {
             // Trivial cases; null to null, instance of type itself returned as is
@@ -165,7 +181,8 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
 
     protected T _deserializeEmbedded(Object ob, DeserializationContext ctxt) throws IOException {
         // default impl: error out
-        ctxt.reportMappingException("Don't know how to convert embedded Object of type %s into %s",
+        ctxt.reportInputMismatch(this,
+                "Don't know how to convert embedded Object of type %s into %s",
                 ob.getClass().getName(), _valueClass.getName());
         return null;
     }
@@ -283,7 +300,8 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
             case STD_STRING_BUILDER:
                 return new StringBuilder(value);
             }
-            throw new IllegalArgumentException();
+            VersionUtil.throwInternal();
+            return null;
         }
 
         @Override
