@@ -4,8 +4,12 @@ import java.util.*;
 
 import com.fasterxml.jackson.annotation.JsonInclude;
 import com.fasterxml.jackson.annotation.JsonProperty;
+import com.fasterxml.jackson.annotation.JsonSetter;
+import com.fasterxml.jackson.annotation.Nulls;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.cfg.ConfigOverride;
 import com.fasterxml.jackson.databind.cfg.MapperConfig;
+import com.fasterxml.jackson.databind.type.TypeFactory;
 import com.fasterxml.jackson.databind.util.ClassUtil;
 
 /**
@@ -16,6 +20,15 @@ public class POJOPropertyBuilder
     extends BeanPropertyDefinition
     implements Comparable<POJOPropertyBuilder>
 {
+    /**
+     * Marker value used to denote that no reference-property information found for
+     * this property
+     *
+     * @since 2.9
+     */
+    private final static AnnotationIntrospector.ReferenceProperty NOT_REFEFERENCE_PROP =
+            AnnotationIntrospector.ReferenceProperty.managed("");
+
     /**
      * Whether property is being composed for serialization
      * (true) or deserialization (false)
@@ -47,6 +60,16 @@ public class POJOPropertyBuilder
 
     protected Linked<AnnotatedMethod> _setters;
 
+    protected transient PropertyMetadata _metadata;
+
+    /**
+     * Lazily accessed information about this property iff it is a forward or
+     * back reference.
+     *
+     * @since 2.9
+     */
+    protected transient AnnotationIntrospector.ReferenceProperty _referenceInfo;
+
     public POJOPropertyBuilder(MapperConfig<?> config, AnnotationIntrospector ai,
             boolean forSerialization, PropertyName internalName) {
         this(config, ai, forSerialization, internalName, internalName);
@@ -62,7 +85,8 @@ public class POJOPropertyBuilder
         _forSerialization = forSerialization;
     }
 
-    public POJOPropertyBuilder(POJOPropertyBuilder src, PropertyName newName)
+    // protected since 2.9 (was public before)
+    protected POJOPropertyBuilder(POJOPropertyBuilder src, PropertyName newName)
     {
         _config = src._config;
         _annotationIntrospector = src._annotationIntrospector;
@@ -74,10 +98,10 @@ public class POJOPropertyBuilder
         _setters = src._setters;
         _forSerialization = src._forSerialization;
     }
-    
+
     /*
     /**********************************************************
-    /* Fluent factory methods
+    /* Mutant factory methods
     /**********************************************************
      */
 
@@ -92,7 +116,7 @@ public class POJOPropertyBuilder
         PropertyName newName = _name.withSimpleName(newSimpleName);
         return (newName == _name) ? this : new POJOPropertyBuilder(this, newName);
     }
-    
+
     /*
     /**********************************************************
     /* Comparable implementation: sort alphabetically, except
@@ -167,7 +191,11 @@ public class POJOPropertyBuilder
         return _anyExplicits(_fields)
                 || _anyExplicits(_getters)
                 || _anyExplicits(_setters)
-                || _anyExplicits(_ctorParameters)
+                // 16-Jan-2016, tatu: Creator names are special, in that name should exist too;
+                //   reason for this is [databind#1317]. Let's hope this works well, may need
+                //   to tweak further if this lowers visibility
+//                || _anyExplicits(_ctorParameters)
+                || _anyExplicitNames(_ctorParameters)
                 ;
     }
 
@@ -179,7 +207,157 @@ public class POJOPropertyBuilder
                 || _anyExplicitNames(_ctorParameters)
                 ;
     }
-    
+
+    /*
+    /**********************************************************
+    /* Simple metadata
+    /**********************************************************
+     */
+
+    @Override
+    public PropertyMetadata getMetadata() {
+        if (_metadata == null) {
+            final Boolean b = _findRequired();
+            final String desc = _findDescription();
+            final Integer idx = _findIndex();
+            final String def = _findDefaultValue();
+            if (b == null && idx == null && def == null) {
+                _metadata = (desc == null) ? PropertyMetadata.STD_REQUIRED_OR_OPTIONAL
+                        : PropertyMetadata.STD_REQUIRED_OR_OPTIONAL.withDescription(desc);
+            } else {
+                _metadata = PropertyMetadata.construct(b, desc, idx, def);
+            }
+            if (!_forSerialization) {
+                _metadata = _getSetterInfo(_metadata);
+            }
+        }
+        return _metadata;
+    }
+
+    /**
+     * Helper method that contains logic for accessing and merging all setter
+     * information that we needed, regarding things like possible merging
+     * of property value, and handling of incoming nulls.
+     */
+    protected PropertyMetadata _getSetterInfo(PropertyMetadata metadata)
+    {
+        boolean needMerge = true;
+        Nulls valueNulls = null;
+        Nulls contentNulls = null;
+        
+        // Slightly confusing: first, annotations should be accessed via primary member
+        // (mutator); but accessor is needed for actual merge operation. So:
+        AnnotatedMember prim = getPrimaryMember();
+        AnnotatedMember acc = getAccessor();
+
+        if (prim != null) {
+            // Ok, first: does property itself have something to say?
+            if (_annotationIntrospector != null) {
+                if (acc != null) {
+                    Boolean b = _annotationIntrospector.findMergeInfo(prim);
+                    if (b != null) {
+                        needMerge = false;
+                        if (b.booleanValue()) {
+                            metadata = metadata.withMergeInfo(PropertyMetadata.MergeInfo.createForPropertyOverride(acc));
+                        }
+                    }
+                }
+                JsonSetter.Value setterInfo = _annotationIntrospector.findSetterInfo(prim);
+                if (setterInfo != null) {
+                    valueNulls = setterInfo.nonDefaultValueNulls();
+                    contentNulls = setterInfo.nonDefaultContentNulls();
+                }
+            }
+            // If not, config override?
+            // 25-Oct-2016, tatu: Either this, or type of accessor...
+            if (needMerge || (valueNulls == null) || (contentNulls == null)) {
+                Class<?> rawType = getRawPrimaryType();
+                ConfigOverride co = _config.getConfigOverride(rawType);
+                JsonSetter.Value setterInfo = co.getSetterInfo();
+                if (setterInfo != null) {
+                    if (valueNulls == null) {
+                        valueNulls = setterInfo.nonDefaultValueNulls();
+                    }
+                    if (contentNulls == null) {
+                        contentNulls = setterInfo.nonDefaultContentNulls();
+                    }
+                }
+                if (needMerge && (acc != null)) {
+                    Boolean b = co.getMergeable();
+                    if (b != null) {
+                        needMerge = false;
+                        if (b.booleanValue()) {
+                            metadata = metadata.withMergeInfo(PropertyMetadata.MergeInfo.createForTypeOverride(acc));
+                        }
+                    }
+                }
+            }
+        }
+        if (needMerge || (valueNulls == null) || (contentNulls == null)) {
+            JsonSetter.Value setterInfo = _config.getDefaultSetterInfo();
+            if (valueNulls == null) {
+                valueNulls = setterInfo.nonDefaultValueNulls();
+            }
+            if (contentNulls == null) {
+                contentNulls = setterInfo.nonDefaultContentNulls();
+            }
+            if (needMerge) {
+                Boolean b = _config.getDefaultMergeable();
+                if (Boolean.TRUE.equals(b) && (acc != null)) {
+                    metadata = metadata.withMergeInfo(PropertyMetadata.MergeInfo.createForDefaults(acc));
+                }
+            }
+        }
+        if ((valueNulls != null) || (contentNulls != null)) {
+            metadata = metadata.withNulls(valueNulls, contentNulls);
+        }
+        return metadata;
+    }
+
+    /**
+     * Type determined from the primary member for the property being built,
+     * considering precedence according to whether we are processing serialization
+     * or deserialization.
+     */
+    @Override
+    public JavaType getPrimaryType() {
+        if (_forSerialization) {
+            AnnotatedMember m = getGetter();
+            if (m == null) {
+                m = getField();
+                if (m == null) {
+                    // 09-Feb-2017, tatu: Not sure if this or `null` but...
+                    return TypeFactory.unknownType();
+                }
+                return m.getType();
+            }
+            return m.getType();
+        }
+        AnnotatedMember m = getConstructorParameter();
+        if (m == null) {
+            m = getSetter();
+            // Important: can't try direct type access for setter; what we need is
+            // type of the first parameter
+            if (m != null) {
+                return ((AnnotatedMethod) m).getParameterType(0);
+            }
+            m = getField();
+        }
+        // for setterless properties, however, can further try getter
+        if (m == null) {
+            m = getGetter();
+            if (m == null) {
+                return TypeFactory.unknownType();
+            }
+        }
+        return m.getType();
+    }
+
+    @Override
+    public Class<?> getRawPrimaryType() {
+        return getPrimaryType().getRawClass();
+    }
+
     /*
     /**********************************************************
     /* BeanPropertyDefinition implementation, accessor access
@@ -384,45 +562,18 @@ public class POJOPropertyBuilder
         }
         return new MemberIterator<AnnotatedParameter>(_ctorParameters);
     }
-    
-    @Override
-    public AnnotatedMember getAccessor()
-    {
-        AnnotatedMember m = getGetter();
-        if (m == null) {
-            m = getField();
-        }
-        return m;
-    }
-
-    @Override
-    public AnnotatedMember getMutator()
-    {
-        AnnotatedMember m = getConstructorParameter();
-        if (m == null) {
-            m = getSetter();
-            if (m == null) {
-                m = getField();
-            }
-        }
-        return m;
-    }
-
-    @Override
-    public AnnotatedMember getNonConstructorMutator() {
-        AnnotatedMember m = getSetter();
-        if (m == null) {
-            m = getField();
-        }
-        return m;
-    }
 
     @Override
     public AnnotatedMember getPrimaryMember() {
         if (_forSerialization) {
             return getAccessor();
         }
-        return getMutator();
+        AnnotatedMember m = getMutator();
+        // for setterless properties, however...
+        if (m == null) {
+            m = getAccessor();
+        }
+        return m;
     }
 
     protected int _getterPriority(AnnotatedMethod m)
@@ -467,12 +618,23 @@ public class POJOPropertyBuilder
 
     @Override
     public AnnotationIntrospector.ReferenceProperty findReferenceType() {
-        return fromMemberAnnotations(new WithMember<AnnotationIntrospector.ReferenceProperty>() {
+        // 30-Mar-2017, tatu: Access lazily but retain information since it needs
+        //   to be accessed multiple times during processing.
+        AnnotationIntrospector.ReferenceProperty result = _referenceInfo;
+        if (result != null) {
+            if (result == NOT_REFEFERENCE_PROP) {
+                return null;
+            }
+            return result;
+        }
+        result = fromMemberAnnotations(new WithMember<AnnotationIntrospector.ReferenceProperty>() {
             @Override
             public AnnotationIntrospector.ReferenceProperty withMember(AnnotatedMember member) {
                 return _annotationIntrospector.findReferenceType(member);
             }
         });
+        _referenceInfo = (result == null) ? NOT_REFEFERENCE_PROP : result;
+        return result;
     }
 
     @Override
@@ -486,19 +648,6 @@ public class POJOPropertyBuilder
         return (b != null) && b.booleanValue();
     }
 
-    @Override
-    public PropertyMetadata getMetadata() {
-        final Boolean b = _findRequired();
-        final String desc = _findDescription();
-        final Integer idx = _findIndex();
-        final String def = _findDefaultValue();
-        if (b == null && idx == null && def == null) {
-            return (desc == null) ? PropertyMetadata.STD_REQUIRED_OR_OPTIONAL
-                    : PropertyMetadata.STD_REQUIRED_OR_OPTIONAL.withDescription(desc);
-        }
-        return PropertyMetadata.construct(b.booleanValue(), desc, idx, def);
-    }
-
     protected Boolean _findRequired() {
        return fromMemberAnnotations(new WithMember<Boolean>() {
             @Override
@@ -569,7 +718,7 @@ public class POJOPropertyBuilder
             }
         }, JsonProperty.Access.AUTO);
     }
-    
+
     /*
     /**********************************************************
     /* Data aggregation
@@ -637,7 +786,7 @@ public class POJOPropertyBuilder
      * @param inferMutators Whether mutators can be "pulled in" by visible
      *    accessors or not. 
      */
-    public void removeNonVisible(boolean inferMutators)
+    public JsonProperty.Access removeNonVisible(boolean inferMutators)
     {
         /* 07-Jun-2015, tatu: With 2.6, we will allow optional definition
          *  of explicit access type for property; if not "AUTO", it will
@@ -667,7 +816,7 @@ public class POJOPropertyBuilder
             }
             break;
         default:
-        case AUTO: // the default case: base it imply on visibility
+        case AUTO: // the default case: base it on visibility
             _getters = _removeNonVisible(_getters);
             _ctorParameters = _removeNonVisible(_ctorParameters);
     
@@ -676,6 +825,7 @@ public class POJOPropertyBuilder
                 _setters = _removeNonVisible(_setters);
             }
         }
+        return acc;
     }
 
     /**
@@ -731,7 +881,7 @@ public class POJOPropertyBuilder
         AnnotationMap ann = _getAllAnnotations(nodes[index]);
         while (++index < nodes.length) {
             if (nodes[index] != null) {
-              return AnnotationMap.merge(ann, _mergeAnnotations(index, nodes));
+                return AnnotationMap.merge(ann, _mergeAnnotations(index, nodes));
             }
         }
         return ann;
@@ -1068,7 +1218,7 @@ public class POJOPropertyBuilder
         }
         return null;
     }
-    
+
     /*
     /**********************************************************
     /* Helper classes
@@ -1135,7 +1285,7 @@ public class POJOPropertyBuilder
 
             if (explName) {
                 if (this.name == null) { // sanity check to catch internal problems
-                    throw new IllegalArgumentException("Can not pass true for 'explName' if name is null/empty");
+                    throw new IllegalArgumentException("Cannot pass true for 'explName' if name is null/empty");
                 }
                 // 03-Apr-2014, tatu: But how about name-space only override?
                 //   Probably should not be explicit? Or, need to merge somehow?
@@ -1223,8 +1373,8 @@ public class POJOPropertyBuilder
         
         @Override
         public String toString() {
-            String msg = value.toString()+"[visible="+isVisible+",ignore="+isMarkedIgnored
-                    +",explicitName="+isNameExplicit+"]";
+            String msg = String.format("%s[visible=%b,ignore=%b,explicitName=%b]",
+                    value.toString(), isVisible, isMarkedIgnored, isNameExplicit);
             if (next != null) {
                 msg = msg + ", "+next.toString();
             }
