@@ -1,14 +1,15 @@
 package com.fasterxml.jackson.databind.ser;
 
-import java.lang.reflect.Method;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.nio.ByteBuffer;
 import java.util.*;
+import java.util.concurrent.atomic.AtomicReference;
 
 import com.fasterxml.jackson.annotation.JsonFormat;
+import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
 import com.fasterxml.jackson.annotation.JsonInclude;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonSerialize;
@@ -56,7 +57,7 @@ public abstract class BasicSerializerFactory
      * not instances
      */
     protected final static HashMap<String, Class<? extends JsonSerializer<?>>> _concreteLazy;
-    
+
     static {
         HashMap<String, Class<? extends JsonSerializer<?>>> concLazy
             = new HashMap<String, Class<? extends JsonSerializer<?>>>();
@@ -93,12 +94,10 @@ public abstract class BasicSerializerFactory
             Object value = en.getValue();
             if (value instanceof JsonSerializer<?>) {
                 concrete.put(en.getKey().getName(), (JsonSerializer<?>) value);
-            } else if (value instanceof Class<?>) {
+            } else {
                 @SuppressWarnings("unchecked")
                 Class<? extends JsonSerializer<?>> cls = (Class<? extends JsonSerializer<?>>) value;
                 concLazy.put(en.getKey().getName(), cls);
-            } else { // should never happen, but:
-                throw new IllegalStateException("Internal error: unrecognized value of type "+en.getClass().getName());
             }
         }
 
@@ -225,16 +224,16 @@ public abstract class BasicSerializerFactory
                 // As per [databind#47], also need to support @JsonValue
                 if (ser == null) {
                     beanDesc = config.introspect(keyType);
-                    AnnotatedMethod am = beanDesc.findJsonValueMethod();
+                    AnnotatedMember am = beanDesc.findJsonValueAccessor();
                     if (am != null) {
-                        final Class<?> rawType = am.getRawReturnType();
+                        final Class<?> rawType = am.getRawType();
                         JsonSerializer<?> delegate = StdKeySerializers.getStdKeySerializer(config,
                                 rawType, true);
-                        Method m = am.getAnnotated();
                         if (config.canOverrideAccessModifiers()) {
-                            ClassUtil.checkAndFixAccess(m, config.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS));
+                            ClassUtil.checkAndFixAccess(am.getMember(),
+                                    config.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS));
                         }
-                        ser = new JsonValueSerializer(m, delegate);
+                        ser = new JsonValueSerializer(am, delegate);
                     } else {
                         ser = StdKeySerializers.getFallbackKeySerializer(config, keyType.getRawClass());
                     }
@@ -309,12 +308,10 @@ public abstract class BasicSerializerFactory
         if (ser == null) {
             Class<? extends JsonSerializer<?>> serClass = _concreteLazy.get(clsName);
             if (serClass != null) {
-                try {
-                    return serClass.newInstance();
-                } catch (Exception e) {
-                    throw new IllegalStateException("Failed to instantiate standard serializer (of type "+serClass.getName()+"): "
-                            +e.getMessage(), e);
-                }
+                // 07-Jan-2017, tatu: Should never fail (since we control constructors),
+                //   but if it does will throw `IllegalArgumentException` with description,
+                //   which we could catch, re-title.
+                return ClassUtil.createInstance(serClass, false);
             }
         }
         return ser;
@@ -346,14 +343,14 @@ public abstract class BasicSerializerFactory
             return SerializableSerializer.instance;
         }
         // Second: @JsonValue for any type
-        AnnotatedMethod valueMethod = beanDesc.findJsonValueMethod();
-        if (valueMethod != null) {
-            Method m = valueMethod.getAnnotated();
+        AnnotatedMember valueAccessor = beanDesc.findJsonValueAccessor();
+        if (valueAccessor != null) {
             if (prov.canOverrideAccessModifiers()) {
-                ClassUtil.checkAndFixAccess(m, prov.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS));
+                ClassUtil.checkAndFixAccess(valueAccessor.getMember(),
+                        prov.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS));
             }
-            JsonSerializer<Object> ser = findSerializerFromAnnotation(prov, valueMethod);
-            return new JsonValueSerializer(m, ser);
+            JsonSerializer<Object> ser = findSerializerFromAnnotation(prov, valueAccessor);
+            return new JsonValueSerializer(valueAccessor, ser);
         }
         // No well-known annotations...
         return null;
@@ -388,17 +385,11 @@ public abstract class BasicSerializerFactory
         if (Map.Entry.class.isAssignableFrom(raw)) {
             // 18-Oct-2015, tatu: With 2.7, need to dig type info:
             JavaType mapEntryType = type.findSuperType(Map.Entry.class);
-            
+
             // 28-Apr-2015, tatu: TypeFactory does it all for us already so
-            JavaType kt = mapEntryType.containedType(0);
-            if (kt == null) {
-                kt = TypeFactory.unknownType();
-            }
-            JavaType vt = mapEntryType.containedType(1);
-            if (vt == null) {
-                vt = TypeFactory.unknownType();
-            }
-            return buildMapEntrySerializer(prov.getConfig(), type, beanDesc, staticTyping, kt, vt);
+            JavaType kt = mapEntryType.containedTypeOrUnknown(0);
+            JavaType vt = mapEntryType.containedTypeOrUnknown(1);
+            return buildMapEntrySerializer(prov, type, beanDesc, staticTyping, kt, vt);
         }
         if (ByteBuffer.class.isAssignableFrom(raw)) {
             return new ByteBufferSerializer();
@@ -549,7 +540,7 @@ public abstract class BasicSerializerFactory
          *   leave it as is; no clean way to make it work.
          */
         if (!staticTyping && type.useStaticType()) {
-            if (!type.isContainerType() || type.getContentType().getRawClass() != Object.class) {
+            if (!type.isContainerType() || !type.getContentType().isJavaLangObject()) {
                 staticTyping = true;
             }
         }
@@ -559,7 +550,7 @@ public abstract class BasicSerializerFactory
         TypeSerializer elementTypeSerializer = createTypeSerializer(config,
                 elementType);
 
-        // if elements have type serializer, can not force static typing:
+        // if elements have type serializer, cannot force static typing:
         if (elementTypeSerializer != null) {
             staticTyping = false;
         }
@@ -665,7 +656,7 @@ public abstract class BasicSerializerFactory
                 // We may also want to use serialize Collections "as beans", if (and only if)
                 // this is specified with `@JsonFormat(shape=Object)`
                 JsonFormat.Value format = beanDesc.findExpectedFormat(null);
-                if (format != null && format.getShape() == JsonFormat.Shape.OBJECT) {
+                if ((format != null) && format.getShape() == JsonFormat.Shape.OBJECT) {
                     return null;
                 }
                 Class<?> raw = type.getRawClass();
@@ -682,7 +673,7 @@ public abstract class BasicSerializerFactory
                     if (isIndexedList(raw)) {
                         if (elementRaw == String.class) {
                             // [JACKSON-829] Must NOT use if we have custom serializer
-                            if (elementValueSerializer == null || ClassUtil.isJacksonStdImpl(elementValueSerializer)) {
+                            if (ClassUtil.isJacksonStdImpl(elementValueSerializer)) {
                                 ser = IndexedStringListSerializer.instance;
                             }
                         } else {
@@ -691,7 +682,7 @@ public abstract class BasicSerializerFactory
                         }
                     } else if (elementRaw == String.class) {
                         // [JACKSON-829] Must NOT use if we have custom serializer
-                        if (elementValueSerializer == null || ClassUtil.isJacksonStdImpl(elementValueSerializer)) {
+                        if (ClassUtil.isJacksonStdImpl(elementValueSerializer)) {
                             ser = StringCollectionSerializer.instance;
                         }
                     }
@@ -726,6 +717,7 @@ public abstract class BasicSerializerFactory
             boolean staticTyping, TypeSerializer vts, JsonSerializer<Object> valueSerializer) {
         return new IndexedListSerializer(elemType, staticTyping, vts, valueSerializer);
     }
+
     public ContainerSerializer<?> buildCollectionSerializer(JavaType elemType,
             boolean staticTyping, TypeSerializer vts, JsonSerializer<Object> valueSerializer) {
         return new CollectionSerializer(elemType, staticTyping, vts, valueSerializer);
@@ -740,7 +732,7 @@ public abstract class BasicSerializerFactory
     /* Factory methods, for Maps
     /**********************************************************
      */
-    
+
     /**
      * Helper method that handles configuration details when constructing serializers for
      * {@link java.util.Map} types.
@@ -751,7 +743,13 @@ public abstract class BasicSerializerFactory
             TypeSerializer elementTypeSerializer, JsonSerializer<Object> elementValueSerializer)
         throws JsonMappingException
     {
-        final SerializationConfig config = prov.getConfig();
+        // [databind#467]: This is where we could allow serialization "as POJO": But! It's
+        // nasty to undo, and does not apply on per-property basis. So, hardly optimal
+        JsonFormat.Value format = beanDesc.findExpectedFormat(null);
+        if ((format != null) && format.getShape() == JsonFormat.Shape.OBJECT) {
+            return null;
+        }
+
         JsonSerializer<?> ser = null;
 
         // Order of lookups:
@@ -759,6 +757,7 @@ public abstract class BasicSerializerFactory
         // 2. Annotations (@JsonValue, @JsonDeserialize)
         // 3. Defaults
         
+        final SerializationConfig config = prov.getConfig();
         for (Serializers serializers : customSerializers()) { // (1) Custom
             ser = serializers.findMapSerializer(config, type, beanDesc,
                     keySerializer, elementTypeSerializer, elementValueSerializer);
@@ -768,16 +767,18 @@ public abstract class BasicSerializerFactory
             ser = findSerializerByAnnotations(prov, type, beanDesc); // (2) Annotations
             if (ser == null) {
                 Object filterId = findFilterId(config, beanDesc);
-                AnnotationIntrospector ai = config.getAnnotationIntrospector();
-                MapSerializer mapSer = MapSerializer.construct(ai.findPropertiesToIgnore(beanDesc.getClassInfo(), true),
+                // 01-May-2016, tatu: Which base type to use here gets tricky, since
+                //   most often it ought to be `Map` or `EnumMap`, but due to abstract
+                //   mapping it will more likely be concrete type like `HashMap`.
+                //   So, for time being, just pass `Map.class`
+                JsonIgnoreProperties.Value ignorals = config.getDefaultPropertyIgnorals(Map.class,
+                        beanDesc.getClassInfo());
+                Set<String> ignored = (ignorals == null) ? null
+                        : ignorals.findIgnoredForSerialization();
+                MapSerializer mapSer = MapSerializer.construct(ignored,
                         type, staticTyping, elementTypeSerializer,
                         keySerializer, elementValueSerializer, filterId);
-                Object suppressableValue = findSuppressableContentValue(config,
-                        type.getContentType(), beanDesc);
-                if (suppressableValue != null) {
-                    mapSer = mapSer.withContentInclusion(suppressableValue);
-                }
-                ser = mapSer;
+                ser = _checkMapContentInclusion(prov, beanDesc, mapSer);
             }
         }
         // [databind#120]: Allow post-processing
@@ -790,40 +791,176 @@ public abstract class BasicSerializerFactory
     }
 
     /**
-     *<p>
-     * NOTE: although return type is left opaque, it really needs to be
-     * <code>JsonInclude.Include</code> for things to work as expected.
+     * Helper method that does figures out content inclusion value to use, if any,
+     * and construct re-configured {@link MapSerializer} appropriately.
      *
-     * @since 2.5
+     * @since 2.9
      */
-    protected Object findSuppressableContentValue(SerializationConfig config,
-            JavaType contentType, BeanDescription beanDesc)
+    @SuppressWarnings("deprecation")
+    protected MapSerializer _checkMapContentInclusion(SerializerProvider prov,
+            BeanDescription beanDesc, MapSerializer mapSer)
         throws JsonMappingException
     {
-        JsonInclude.Value inclV = beanDesc.findPropertyInclusion(config.getDefaultPropertyInclusion());
-        
-        if (inclV == null) {
-            return null;
+        final JavaType contentType = mapSer.getContentType();
+        JsonInclude.Value inclV = _findInclusionWithContent(prov, beanDesc,
+                contentType, Map.class);
+
+        // Need to support global legacy setting, for now:
+        JsonInclude.Include incl = (inclV == null) ? JsonInclude.Include.USE_DEFAULTS : inclV.getContentInclusion();
+        if (incl == JsonInclude.Include.USE_DEFAULTS
+                || incl == JsonInclude.Include.ALWAYS) {
+            if (!prov.isEnabled(SerializationFeature.WRITE_NULL_MAP_VALUES)) {
+                return mapSer.withContentInclusion(null, true);
+            }
+            return mapSer;
         }
-        JsonInclude.Include incl = inclV.getContentInclusion();
+
+        // NOTE: mostly copied from `PropertyBuilder`; would be nice to refactor
+        // but code is not identical nor are these types related
+        Object valueToSuppress;
+        boolean suppressNulls = true; // almost always, but possibly not with CUSTOM
+
         switch (incl) {
-        case USE_DEFAULTS: // means "dunno"
-            return null;
         case NON_DEFAULT:
-            // 19-Oct-2014, tatu: Not sure what this'd mean; so take it to mean "NON_EMPTY"...
-            // 11-Nov-2015, tatu: With 2.6, we did indeed revert to "NON_EMPTY", but that did
-            //    not go well, so with 2.7, we'll do this instead...
-            //   But not 100% sure if we ought to call new `JsonSerializer.findDefaultValue()`;
-            //   to do that, would need to locate said serializer
-//            incl = JsonInclude.Include.NON_EMPTY;
+            valueToSuppress = BeanUtil.getDefaultValue(contentType);
+            if (valueToSuppress != null) {
+                if (valueToSuppress.getClass().isArray()) {
+                    valueToSuppress = ArrayBuilders.getArrayComparator(valueToSuppress);
+                }
+            }
             break;
-        default:
-            // all other modes actually good as is, unless we'll find better ways
+        case NON_ABSENT: // new with 2.6, to support Guava/JDK8 Optionals
+            // and for referential types, also "empty", which in their case means "absent"
+            valueToSuppress = contentType.isReferenceType()
+                    ? MapSerializer.MARKER_FOR_EMPTY : null;
+            break;
+        case NON_EMPTY:
+            valueToSuppress = MapSerializer.MARKER_FOR_EMPTY;
+            break;
+        case CUSTOM: // new with 2.9
+            valueToSuppress = prov.includeFilterInstance(null, inclV.getContentFilter());
+            if (valueToSuppress == null) { // is this legal?
+                suppressNulls = true;
+            } else {
+                suppressNulls = prov.includeFilterSuppressNulls(valueToSuppress);
+            }
+            break;
+        case NON_NULL:
+        default: // should not matter but...
+            valueToSuppress = null;
             break;
         }
-        return incl;
+        return mapSer.withContentInclusion(valueToSuppress, suppressNulls);
     }
 
+    /**
+     * @since 2.9
+     */
+    protected JsonSerializer<?> buildMapEntrySerializer(SerializerProvider prov,
+            JavaType type, BeanDescription beanDesc, boolean staticTyping,
+            JavaType keyType, JavaType valueType)
+        throws JsonMappingException
+    {
+        // [databind#865]: Allow serialization "as POJO" -- note: to undo, declare
+        //   serialization as `Shape.NATURAL` instead; that's JSON Object too.
+        JsonFormat.Value formatOverride = prov.getDefaultPropertyFormat(Map.Entry.class);
+        JsonFormat.Value formatFromAnnotation = beanDesc.findExpectedFormat(null);
+        JsonFormat.Value format = JsonFormat.Value.merge(formatFromAnnotation, formatOverride);
+        if (format.getShape() == JsonFormat.Shape.OBJECT) {
+            return null;
+        }
+        MapEntrySerializer ser = new MapEntrySerializer(valueType, keyType, valueType,
+                staticTyping, createTypeSerializer(prov.getConfig(), valueType), null);
+
+        final JavaType contentType = ser.getContentType();
+        JsonInclude.Value inclV = _findInclusionWithContent(prov, beanDesc,
+                contentType, Map.Entry.class);
+
+        // Need to support global legacy setting, for now:
+        JsonInclude.Include incl = (inclV == null) ? JsonInclude.Include.USE_DEFAULTS : inclV.getContentInclusion();
+        if (incl == JsonInclude.Include.USE_DEFAULTS
+                || incl == JsonInclude.Include.ALWAYS) {
+            return ser;
+        }
+
+        // NOTE: mostly copied from `PropertyBuilder`; would be nice to refactor
+        // but code is not identical nor are these types related
+        Object valueToSuppress;
+        boolean suppressNulls = true; // almost always, but possibly not with CUSTOM
+
+        switch (incl) {
+        case NON_DEFAULT:
+            valueToSuppress = BeanUtil.getDefaultValue(contentType);
+            if (valueToSuppress != null) {
+                if (valueToSuppress.getClass().isArray()) {
+                    valueToSuppress = ArrayBuilders.getArrayComparator(valueToSuppress);
+                }
+            }
+            break;
+        case NON_ABSENT:
+            valueToSuppress = contentType.isReferenceType()
+                    ? MapSerializer.MARKER_FOR_EMPTY : null;
+            break;
+        case NON_EMPTY:
+            valueToSuppress = MapSerializer.MARKER_FOR_EMPTY;
+            break;
+        case CUSTOM:
+            valueToSuppress = prov.includeFilterInstance(null, inclV.getContentFilter());
+            if (valueToSuppress == null) { // is this legal?
+                suppressNulls = true;
+            } else {
+                suppressNulls = prov.includeFilterSuppressNulls(valueToSuppress);
+            }
+            break;
+        case NON_NULL:
+        default: // should not matter but...
+            valueToSuppress = null;
+            break;
+        }
+        return ser.withContentInclusion(valueToSuppress, suppressNulls);
+    }
+
+    /**
+     * Helper method used for finding inclusion definitions for structured
+     * container types like <code>Map</code>s and referential types
+     * (like <code>AtomicReference</code>).
+     *
+     * @param contentType Declared full content type of container
+     * @param configType Raw base type under which `configOverride`, if any, needs to be defined
+     */
+    protected JsonInclude.Value _findInclusionWithContent(SerializerProvider prov,
+            BeanDescription beanDesc,
+            JavaType contentType, Class<?> configType)
+        throws JsonMappingException
+    {
+        final SerializationConfig config = prov.getConfig();
+
+        // Defaulting gets complicated because we might have two distinct
+        //   axis to consider: Container type itself , and then value (content) type.
+        //  Start with Container-defaults, then use more-specific value override, if any.
+
+        // Start by getting global setting, overridden by Map-type-override
+        JsonInclude.Value inclV = beanDesc.findPropertyInclusion(config.getDefaultPropertyInclusion());
+        inclV = config.getDefaultPropertyInclusion(configType, inclV);
+
+        // and then merge content-type overrides, if any. But note that there's
+        // content-to-value inclusion shift we have to do
+        JsonInclude.Value valueIncl = config.getDefaultPropertyInclusion(contentType.getRawClass(), null);
+
+        if (valueIncl != null) {
+            switch (valueIncl.getValueInclusion()) {
+            case USE_DEFAULTS:
+                break;
+            case CUSTOM:
+                inclV = inclV.withContentFilter(valueIncl.getContentFilter());
+                break;
+            default:
+                inclV = inclV.withContentInclusion(valueIncl.getValueInclusion());
+            }
+        }
+        return inclV;
+    }
+    
     /*
     /**********************************************************
     /* Factory methods, for Arrays
@@ -841,7 +978,7 @@ public abstract class BasicSerializerFactory
         throws JsonMappingException
     {
         // 25-Jun-2015, tatu: Note that unlike with Collection(Like) and Map(Like) types, array
-        //   types can not be annotated (in theory I guess we could have mix-ins but... ?)
+        //   types cannot be annotated (in theory I guess we could have mix-ins but... ?)
         //   so we need not do primary annotation lookup here.
         //   So all we need is (1) Custom, (2) Default array serializers
         SerializationConfig config = prov.getConfig();
@@ -880,6 +1017,96 @@ public abstract class BasicSerializerFactory
          return ser;
     }
 
+    /*
+    /**********************************************************
+    /* Factory methods for Reference types
+    /* (demoted from BeanSF down here in 2.9)
+    /**********************************************************
+     */
+
+    /**
+     * @since 2.7
+     */
+    public JsonSerializer<?> findReferenceSerializer(SerializerProvider prov, ReferenceType refType,
+            BeanDescription beanDesc, boolean staticTyping)
+        throws JsonMappingException
+    {
+        JavaType contentType = refType.getContentType(); 
+        TypeSerializer contentTypeSerializer = contentType.getTypeHandler();
+        final SerializationConfig config = prov.getConfig();
+        if (contentTypeSerializer == null) {
+            contentTypeSerializer = createTypeSerializer(config, contentType);
+        }
+        JsonSerializer<Object> contentSerializer = contentType.getValueHandler();
+        for (Serializers serializers : customSerializers()) {
+            JsonSerializer<?> ser = serializers.findReferenceSerializer(config, refType, beanDesc,
+                    contentTypeSerializer, contentSerializer);
+            if (ser != null) {
+                return ser;
+            }
+        }
+        if (refType.isTypeOrSubTypeOf(AtomicReference.class)) {
+            return buildAtomicReferenceSerializer(prov, refType, beanDesc, staticTyping,
+                    contentTypeSerializer, contentSerializer);
+        }
+        return null;
+    }
+
+    protected JsonSerializer<?> buildAtomicReferenceSerializer(SerializerProvider prov,
+            ReferenceType refType, BeanDescription beanDesc, boolean staticTyping,
+            TypeSerializer contentTypeSerializer, JsonSerializer<Object> contentSerializer)
+        throws JsonMappingException
+    {
+        final JavaType contentType = refType.getReferencedType();
+        JsonInclude.Value inclV = _findInclusionWithContent(prov, beanDesc,
+                contentType, AtomicReference.class);
+        
+        // Need to support global legacy setting, for now:
+        JsonInclude.Include incl = (inclV == null) ? JsonInclude.Include.USE_DEFAULTS : inclV.getContentInclusion();
+        Object valueToSuppress;
+        boolean suppressNulls;
+
+        if (incl == JsonInclude.Include.USE_DEFAULTS
+                || incl == JsonInclude.Include.ALWAYS) {
+            valueToSuppress = null;
+            suppressNulls = false;
+        } else {
+            suppressNulls = true;
+            switch (incl) {
+            case NON_DEFAULT:
+                valueToSuppress = BeanUtil.getDefaultValue(contentType);
+                if (valueToSuppress != null) {
+                    if (valueToSuppress.getClass().isArray()) {
+                        valueToSuppress = ArrayBuilders.getArrayComparator(valueToSuppress);
+                    }
+                }
+                break;
+            case NON_ABSENT:
+                valueToSuppress = contentType.isReferenceType()
+                        ? MapSerializer.MARKER_FOR_EMPTY : null;
+                break;
+            case NON_EMPTY:
+                valueToSuppress = MapSerializer.MARKER_FOR_EMPTY;
+                break;
+            case CUSTOM:
+                valueToSuppress = prov.includeFilterInstance(null, inclV.getContentFilter());
+                if (valueToSuppress == null) { // is this legal?
+                    suppressNulls = true;
+                } else {
+                    suppressNulls = prov.includeFilterSuppressNulls(valueToSuppress);
+                }
+                break;
+            case NON_NULL:
+            default: // should not matter but...
+                valueToSuppress = null;
+                break;
+            }
+        }
+        AtomicReferenceSerializer ser = new AtomicReferenceSerializer(refType, staticTyping,
+                contentTypeSerializer, contentSerializer);
+        return ser.withContentInclusion(valueToSuppress, suppressNulls);
+    }
+
     /*
     /**********************************************************
     /* Factory methods, for non-container types
@@ -897,16 +1124,6 @@ public abstract class BasicSerializerFactory
         return new IteratorSerializer(valueType, staticTyping, createTypeSerializer(config, valueType));
     }
 
-    @Deprecated // since 2.5
-    protected JsonSerializer<?> buildIteratorSerializer(SerializationConfig config,
-            JavaType type, BeanDescription beanDesc, boolean staticTyping) throws JsonMappingException
-    {
-        JavaType[] params = config.getTypeFactory().findTypeParameters(type, Iterator.class);
-        JavaType vt = (params == null || params.length != 1) ?
-                TypeFactory.unknownType() : params[0];
-        return buildIteratorSerializer(config, type, beanDesc, staticTyping, vt); 
-    }
-
     /**
      * @since 2.5
      */
@@ -918,30 +1135,6 @@ public abstract class BasicSerializerFactory
         return new IterableSerializer(valueType, staticTyping, createTypeSerializer(config, valueType));
     }
 
-    @Deprecated // since 2.5
-    protected JsonSerializer<?> buildIterableSerializer(SerializationConfig config,
-            JavaType type, BeanDescription beanDesc,
-            boolean staticTyping)
-        throws JsonMappingException
-    {
-        JavaType[] params = config.getTypeFactory().findTypeParameters(type, Iterable.class);
-        JavaType vt = (params == null || params.length != 1) ?
-                TypeFactory.unknownType() : params[0];
-        return buildIterableSerializer(config, type, beanDesc, staticTyping, vt); 
-    }
-    
-    /**
-     * @since 2.5
-     */
-    protected JsonSerializer<?> buildMapEntrySerializer(SerializationConfig config,
-            JavaType type, BeanDescription beanDesc, boolean staticTyping,
-            JavaType keyType, JavaType valueType)
-        throws JsonMappingException
-    {
-        return new MapEntrySerializer(valueType, keyType, valueType,
-                staticTyping, createTypeSerializer(config, valueType), null);
-    }
-
     protected JsonSerializer<?> buildEnumSerializer(SerializationConfig config,
             JavaType type, BeanDescription beanDesc)
         throws JsonMappingException
@@ -961,7 +1154,7 @@ public abstract class BasicSerializerFactory
         @SuppressWarnings("unchecked")
         Class<Enum<?>> enumClass = (Class<Enum<?>>) type.getRawClass();
         JsonSerializer<?> ser = EnumSerializer.construct(enumClass, config, beanDesc, format);
-        // [Issue#120]: Allow post-processing
+        // [databind#120]: Allow post-processing
         if (_factoryConfig.hasSerializerModifiers()) {
             for (BeanSerializerModifier mod : _factoryConfig.serializerModifiers()) {
                 ser = mod.modifyEnumSerializer(config, type, beanDesc, ser);
@@ -1029,7 +1222,7 @@ public abstract class BasicSerializerFactory
     protected boolean usesStaticTyping(SerializationConfig config,
             BeanDescription beanDesc, TypeSerializer typeSer)
     {
-        /* 16-Aug-2010, tatu: If there is a (value) type serializer, we can not force
+        /* 16-Aug-2010, tatu: If there is a (value) type serializer, we cannot force
          *    static typing; that would make it impossible to handle expected subtypes
          */
         if (typeSer != null) {
@@ -1043,6 +1236,8 @@ public abstract class BasicSerializerFactory
         return config.isEnabled(MapperFeature.USE_STATIC_TYPING);
     }
 
+    // Commented out in 2.9
+    /*
     protected Class<?> _verifyAsClass(Object src, String methodName, Class<?> noneClass)
     {
         if (src == null) {
@@ -1057,4 +1252,5 @@ public abstract class BasicSerializerFactory
         }
         return cls;
     }
+    */
 }
