@@ -1,7 +1,6 @@
 package com.fasterxml.jackson.databind.ser;
 
 import java.util.*;
-import java.util.concurrent.atomic.AtomicReference;
 
 import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
 import com.fasterxml.jackson.annotation.ObjectIdGenerator;
@@ -9,7 +8,6 @@ import com.fasterxml.jackson.annotation.ObjectIdGenerators;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.As;
 
 import com.fasterxml.jackson.databind.*;
-import com.fasterxml.jackson.databind.cfg.ConfigOverride;
 import com.fasterxml.jackson.databind.cfg.SerializerFactoryConfig;
 import com.fasterxml.jackson.databind.introspect.*;
 import com.fasterxml.jackson.databind.jsontype.NamedType;
@@ -18,7 +16,6 @@ import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
 import com.fasterxml.jackson.databind.ser.impl.FilteredBeanPropertyWriter;
 import com.fasterxml.jackson.databind.ser.impl.ObjectIdWriter;
 import com.fasterxml.jackson.databind.ser.impl.PropertyBasedObjectIdGenerator;
-import com.fasterxml.jackson.databind.ser.std.AtomicReferenceSerializer;
 import com.fasterxml.jackson.databind.ser.std.MapSerializer;
 import com.fasterxml.jackson.databind.ser.std.StdDelegatingSerializer;
 import com.fasterxml.jackson.databind.type.ReferenceType;
@@ -100,7 +97,7 @@ public class BeanSerializerFactory
          */
         if (getClass() != BeanSerializerFactory.class) {
             throw new IllegalStateException("Subtype of BeanSerializerFactory ("+getClass().getName()
-                    +") has not properly overridden method 'withAdditionalSerializers': can not instantiate subtype with "
+                    +") has not properly overridden method 'withAdditionalSerializers': cannot instantiate subtype with "
                     +"additional serializer definitions");
         }
         return new BeanSerializerFactory(config);
@@ -110,7 +107,7 @@ public class BeanSerializerFactory
     protected Iterable<Serializers> customSerializers() {
         return _factoryConfig.serializers();
     }
-    
+
     /*
     /**********************************************************
     /* SerializerFactory impl
@@ -282,34 +279,6 @@ public class BeanSerializerFactory
         return constructBeanSerializer(prov, beanDesc);
     }
 
-    /**
-     * @since 2.7
-     */
-    public JsonSerializer<?> findReferenceSerializer(SerializerProvider prov, ReferenceType refType,
-            BeanDescription beanDesc, boolean staticTyping)
-        throws JsonMappingException
-    {
-        JavaType contentType = refType.getContentType(); 
-        TypeSerializer contentTypeSerializer = contentType.getTypeHandler();
-        final SerializationConfig config = prov.getConfig();
-        if (contentTypeSerializer == null) {
-            contentTypeSerializer = createTypeSerializer(config, contentType);
-        }
-        JsonSerializer<Object> contentSerializer = contentType.getValueHandler();
-        for (Serializers serializers : customSerializers()) {
-            JsonSerializer<?> ser = serializers.findReferenceSerializer(config, refType, beanDesc,
-                    contentTypeSerializer, contentSerializer);
-            if (ser != null) {
-                return ser;
-            }
-        }
-        if (refType.isTypeOrSubTypeOf(AtomicReference.class)) {
-            return new AtomicReferenceSerializer(refType, staticTyping,
-                    contentTypeSerializer, contentSerializer);
-        }
-        return null;
-    }
-
     /**
      * Method called to create a type information serializer for values of given
      * non-container property
@@ -389,7 +358,7 @@ public class BeanSerializerFactory
         // 05-Jul-2012, tatu: ... but we should be able to just return "unknown type" serializer, right?
         if (beanDesc.getBeanClass() == Object.class) {
             return prov.getUnknownTypeSerializer(Object.class);
-//            throw new IllegalArgumentException("Can not create bean serializer for Object.class");
+//            throw new IllegalArgumentException("Cannot create bean serializer for Object.class");
         }
         final SerializationConfig config = prov.getConfig();
         BeanSerializerBuilder builder = constructBeanSerializerBuilder(beanDesc);
@@ -423,10 +392,9 @@ public class BeanSerializerFactory
             }
         }
 
-        /* And if Object Id is needed, some preparation for that as well: better
-         * do before view handling, mostly for the custom id case which needs
-         * access to a property
-         */
+        // And if Object Id is needed, some preparation for that as well: better
+        // do before view handling, mostly for the custom id case which needs
+        // access to a property
         builder.setObjectIdWriter(constructObjectIdHandler(prov, beanDesc, props));
         
         builder.setProperties(props);
@@ -450,7 +418,7 @@ public class BeanSerializerFactory
             // TODO: can we find full PropertyName?
             PropertyName name = PropertyName.construct(anyGetter.getName());
             BeanProperty.Std anyProp = new BeanProperty.Std(name, valueType, null,
-                    beanDesc.getClassAnnotations(), anyGetter, PropertyMetadata.STD_OPTIONAL);
+                    anyGetter, PropertyMetadata.STD_OPTIONAL);
             builder.setAnyGetter(new AnyGetterWriter(anyProp, anyGetter, anySer));
         }
         // Next: need to gather view information, if any:
@@ -463,7 +431,13 @@ public class BeanSerializerFactory
             }
         }
 
-        JsonSerializer<Object> ser = (JsonSerializer<Object>) builder.build();
+        JsonSerializer<Object> ser = null;
+        try {
+            ser = (JsonSerializer<Object>) builder.build();
+        } catch (RuntimeException e) {
+            prov.reportBadTypeDefinition(beanDesc, "Failed to construct BeanSerializer for %s: (%s) %s",
+                    beanDesc.getType(), e.getClass().getName(), e.getMessage());
+        }
         if (ser == null) {
             // If we get this far, there were no properties found, so no regular BeanSerializer
             // would be constructed. But, couple of exceptions.
@@ -494,14 +468,13 @@ public class BeanSerializerFactory
             for (int i = 0, len = props.size() ;; ++i) {
                 if (i == len) {
                     throw new IllegalArgumentException("Invalid Object Id definition for "+beanDesc.getBeanClass().getName()
-                            +": can not find property with name '"+propName+"'");
+                            +": cannot find property with name '"+propName+"'");
                 }
                 BeanPropertyWriter prop = props.get(i);
                 if (propName.equals(prop.getName())) {
                     idProp = prop;
-                    /* Let's force it to be the first property to output
-                     * (although it may still get rearranged etc)
-                     */
+                    // Let's force it to be the first property to output
+                    // (although it may still get rearranged etc)
                     if (i > 0) {
                         props.remove(i);
                         props.add(0, idProp);
@@ -553,7 +526,7 @@ public class BeanSerializerFactory
     
     /**
      * Helper method used to skip processing for types that we know
-     * can not be (i.e. are never consider to be) beans: 
+     * cannot be (i.e. are never consider to be) beans: 
      * things like primitives, Arrays, Enums, and proxy types.
      *<p>
      * Note that usually we shouldn't really be getting these sort of
@@ -689,6 +662,7 @@ public class BeanSerializerFactory
      * Method that will apply by-type limitations (as per [JACKSON-429]);
      * by default this is based on {@link com.fasterxml.jackson.annotation.JsonIgnoreType}
      * annotation but can be supplied by module-provided introspectors too.
+     * Starting with 2.8 there are also "Config overrides" to consider.
      */
     protected void removeIgnorableTypes(SerializationConfig config, BeanDescription beanDesc,
             List<BeanPropertyDefinition> properties)
@@ -699,18 +673,19 @@ public class BeanSerializerFactory
         while (it.hasNext()) {
             BeanPropertyDefinition property = it.next();
             AnnotatedMember accessor = property.getAccessor();
+            /* 22-Oct-2016, tatu: Looks like this removal is an important part of
+             *    processing, as taking it out will result in a few test failures...
+             *    But should probably be done somewhere else, not here?
+             */
             if (accessor == null) {
                 it.remove();
                 continue;
             }
-            Class<?> type = accessor.getRawType();
+            Class<?> type = property.getRawPrimaryType();
             Boolean result = ignores.get(type);
             if (result == null) {
                 // 21-Apr-2016, tatu: For 2.8, can specify config overrides
-                ConfigOverride override = config.findConfigOverride(type);
-                if (override != null) {
-                    result = override.getIsIgnoredType();
-                }
+                result = config.getConfigOverride(type).getIsIgnoredType();
                 if (result == null) {
                     BeanDescription desc = config.introspectClassAnnotations(type);
                     AnnotatedClass ac = desc.getClassInfo();
@@ -793,7 +768,7 @@ public class BeanSerializerFactory
         final PropertyName name = propDef.getFullName();
         JavaType type = accessor.getType();
         BeanProperty.Std property = new BeanProperty.Std(name, type, propDef.getWrapperName(),
-                pb.getClassAnnotations(), accessor, propDef.getMetadata());
+                accessor, propDef.getMetadata());
 
         // Does member specify a serializer? If so, let's use it.
         JsonSerializer<?> annotatedSerializer = findSerializerFromAnnotation(prov,
