@@ -6,6 +6,7 @@ import java.util.*;
 
 import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.type.WritableTypeId;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
 import com.fasterxml.jackson.databind.introspect.ObjectIdInfo;
@@ -17,6 +18,7 @@ import com.fasterxml.jackson.databind.jsonschema.SchemaAware;
 import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
 import com.fasterxml.jackson.databind.node.ObjectNode;
 import com.fasterxml.jackson.databind.ser.*;
+import com.fasterxml.jackson.databind.ser.impl.MapEntrySerializer;
 import com.fasterxml.jackson.databind.ser.impl.ObjectIdWriter;
 import com.fasterxml.jackson.databind.ser.impl.PropertyBasedObjectIdGenerator;
 import com.fasterxml.jackson.databind.ser.impl.WritableObjectId;
@@ -37,7 +39,7 @@ public abstract class BeanSerializerBase
         JsonFormatVisitable, SchemaAware
 {
     protected final static PropertyName NAME_FOR_OBJECT_REF = new PropertyName("#object-ref");
-    
+
     final protected static BeanPropertyWriter[] NO_PROPS = new BeanPropertyWriter[0];
 
     /*
@@ -46,6 +48,11 @@ public abstract class BeanSerializerBase
     /**********************************************************
      */
 
+    /**
+     * @since 2.9
+     */
+    final protected JavaType _beanType;
+
     /**
      * Writers used for outputting actual property values
      */
@@ -103,6 +110,7 @@ public abstract class BeanSerializerBase
             BeanPropertyWriter[] properties, BeanPropertyWriter[] filteredProperties)
     {
         super(type);
+        _beanType = type;
         _props = properties;
         _filteredProps = filteredProperties;
         if (builder == null) { // mostly for testing
@@ -125,6 +133,7 @@ public abstract class BeanSerializerBase
             BeanPropertyWriter[] properties, BeanPropertyWriter[] filteredProperties)
     {
         super(src._handledType);
+        _beanType = src._beanType;
         _props = properties;
         _filteredProps = filteredProperties;
 
@@ -148,6 +157,7 @@ public abstract class BeanSerializerBase
             ObjectIdWriter objectIdWriter, Object filterId)
     {
         super(src._handledType);
+        _beanType = src._beanType;
         _props = src._props;
         _filteredProps = src._filteredProps;
         
@@ -168,6 +178,7 @@ public abstract class BeanSerializerBase
     {
         super(src._handledType);
 
+        _beanType = src._beanType;
         final BeanPropertyWriter[] propsIn = src._props;
         final BeanPropertyWriter[] fpropsIn = src._filteredProps;
         final int len = propsIn.length;
@@ -318,9 +329,6 @@ public abstract class BeanSerializerBase
                 // It not, we can use declared return type if and only if declared type is final:
                 // if not, we don't really know the actual type until we get the instance.
                 if (type == null) {
-                    // 30-Oct-2015, tatu: Not sure why this was used
-//                    type = provider.constructType(prop.getGenericPropertyType());
-                    // but this looks better
                     type = prop.getType();
                     if (!type.isFinal()) {
                         if (type.isContainerType() || type.containedTypeCount() > 0) {
@@ -346,14 +354,18 @@ public abstract class BeanSerializerBase
                     }
                 }
             }
-            prop.assignSerializer(ser);
-            // and maybe replace filtered property too? (see [JACKSON-364])
+            // and maybe replace filtered property too?
             if (i < filteredCount) {
                 BeanPropertyWriter w2 = _filteredProps[i];
                 if (w2 != null) {
                     w2.assignSerializer(ser);
+                    // 17-Mar-2017, tatu: Typically will lead to chained call to original property,
+                    //    which would lead to double set. Not a problem itself, except... unwrapping
+                    //    may require work to be done, which does lead to an actual issue.
+                    continue;
                 }
             }
+            prop.assignSerializer(ser);
         }
 
         // also, any-getter may need to be resolved
@@ -418,11 +430,27 @@ public abstract class BeanSerializerBase
                     case NUMBER_INT:
                         // 12-Oct-2014, tatu: May need to introspect full annotations... but
                         //   for now, just do class ones
-                        BeanDescription desc = config.introspectClassAnnotations(_handledType);
-                        JsonSerializer<?> ser = EnumSerializer.construct(_handledType,
+                        BeanDescription desc = config.introspectClassAnnotations(_beanType);
+                        JsonSerializer<?> ser = EnumSerializer.construct(_beanType.getRawClass(),
                                 provider.getConfig(), desc, format);
                         return provider.handlePrimaryContextualization(ser, property);
                     }
+                // 16-Oct-2016, tatu: Ditto for `Map`, `Map.Entry` subtypes
+                } else if (shape == JsonFormat.Shape.NATURAL) {
+                    if (_beanType.isMapLikeType() && Map.class.isAssignableFrom(_handledType)) {
+                        ;
+                    } else if (Map.Entry.class.isAssignableFrom(_handledType)) {
+                        JavaType mapEntryType = _beanType.findSuperType(Map.Entry.class);
+
+                        JavaType kt = mapEntryType.containedTypeOrUnknown(0);
+                        JavaType vt = mapEntryType.containedTypeOrUnknown(1);
+
+                        // 16-Oct-2016, tatu: could have problems with type handling, as we do not
+                        //   see if "static" typing is needed, nor look for `TypeSerializer` yet...
+                        JsonSerializer<?> ser = new MapEntrySerializer(_beanType, kt, vt,
+                                false, null, property);
+                        return provider.handlePrimaryContextualization(ser, property);
+                    }
                 }
             }
         }
@@ -441,15 +469,14 @@ public abstract class BeanSerializerBase
             if (objectIdInfo == null) {
                 // no ObjectId override, but maybe ObjectIdRef?
                 if (oiw != null) {
-                    objectIdInfo = intr.findObjectReferenceInfo(accessor,
-                            new ObjectIdInfo(NAME_FOR_OBJECT_REF, null, null, null));
-                    oiw = _objectIdWriter.withAlwaysAsId(objectIdInfo.getAlwaysAsId());
+                    objectIdInfo = intr.findObjectReferenceInfo(accessor, null);
+                    if (objectIdInfo != null) {
+                        oiw = _objectIdWriter.withAlwaysAsId(objectIdInfo.getAlwaysAsId());
+                    }
                 }
             } else {
-                /* Ugh: mostly copied from BeanSerializerBase: but can't easily
-                 * change it to be able to move to SerializerProvider (where it
-                 * really belongs)
-                 */
+                // Ugh: mostly copied from BeanDeserializerBase: but can't easily change it
+                // to be able to move to SerializerProvider (where it really belongs)
                 
                 // 2.1: allow modifications by "id ref" annotations as well:
                 objectIdInfo = intr.findObjectReferenceInfo(accessor, objectIdInfo);
@@ -462,17 +489,17 @@ public abstract class BeanSerializerBase
                     String propName = objectIdInfo.getPropertyName().getSimpleName();
                     BeanPropertyWriter idProp = null;
 
-                    for (int i = 0, len = _props.length ;; ++i) {
+                    for (int i = 0, len = _props.length; ; ++i) {
                         if (i == len) {
-                            throw new IllegalArgumentException("Invalid Object Id definition for "+_handledType.getName()
-                                    +": can not find property with name '"+propName+"'");
+                            provider.reportBadDefinition(_beanType, String.format(
+                                    "Invalid Object Id definition for %s: cannot find property with name '%s'",
+                                    handledType().getName(), propName));
                         }
                         BeanPropertyWriter prop = _props[i];
                         if (propName.equals(prop.getName())) {
                             idProp = prop;
-                            /* Let's force it to be the first property to output
-                             * (although it may still get rearranged etc)
-                             */
+                            // Let's force it to be the first property to output
+                            // (although it may still get rearranged etc)
                             if (i > 0) { // note: must shuffle both regular properties and filtered
                                 System.arraycopy(_props, 0, _props, 1, i);
                                 _props[0] = idProp;
@@ -494,7 +521,6 @@ public abstract class BeanSerializerBase
                             objectIdInfo.getAlwaysAsId());
                 }
             }
-            
             // Or change Filter Id in use?
             Object filterId = intr.findFilterId(accessor);
             if (filterId != null) {
@@ -523,6 +549,7 @@ public abstract class BeanSerializerBase
         if (shape == null) {
             shape = _serializationShape;
         }
+        // last but not least; may need to transmute into as-array serialization
         if (shape == JsonFormat.Shape.ARRAY) {
             return contextual.asArraySerializer();
         }
@@ -568,23 +595,15 @@ public abstract class BeanSerializerBase
             return;
         }
 
-        String typeStr = (_typeId == null) ? null : _customTypeId(bean);
-        if (typeStr == null) {
-            typeSer.writeTypePrefixForObject(bean, gen);
-        } else {
-            typeSer.writeCustomTypePrefixForObject(bean, gen, typeStr);
-        }
         gen.setCurrentValue(bean); // [databind#631]
+        WritableTypeId typeIdDef = _typeIdDef(typeSer, bean, JsonToken.START_OBJECT);
+        typeSer.writeTypePrefix(gen, typeIdDef);
         if (_propertyFilterId != null) {
             serializeFieldsFiltered(bean, gen, provider);
         } else {
             serializeFields(bean, gen, provider);
         }
-        if (typeStr == null) {
-            typeSer.writeTypeSuffixForObject(bean, gen);
-        } else {
-            typeSer.writeCustomTypeSuffixForObject(bean, gen, typeStr);
-        }
+        typeSer.writeTypeSuffix(gen, typeIdDef);
     }
 
     protected final void _serializeWithObjectId(Object bean, JsonGenerator gen, SerializerProvider provider,
@@ -631,33 +650,43 @@ public abstract class BeanSerializerBase
             w.serializer.serialize(id, gen, provider);
             return;
         }
-
         _serializeObjectId(bean, gen, provider, typeSer, objectId);
     }
 
-    protected  void _serializeObjectId(Object bean, JsonGenerator gen,SerializerProvider provider,
+    protected  void _serializeObjectId(Object bean, JsonGenerator g,
+            SerializerProvider provider,
             TypeSerializer typeSer, WritableObjectId objectId) throws IOException
     {
         final ObjectIdWriter w = _objectIdWriter;
-        String typeStr = (_typeId == null) ? null :_customTypeId(bean);
-        if (typeStr == null) {
-            typeSer.writeTypePrefixForObject(bean, gen);
-        } else {
-            typeSer.writeCustomTypePrefixForObject(bean, gen, typeStr);
-        }
-        objectId.writeAsField(gen, provider, w);
+        WritableTypeId typeIdDef = _typeIdDef(typeSer, bean, JsonToken.START_OBJECT);
+
+        typeSer.writeTypePrefix(g, typeIdDef);
+        objectId.writeAsField(g, provider, w);
         if (_propertyFilterId != null) {
-            serializeFieldsFiltered(bean, gen, provider);
+            serializeFieldsFiltered(bean, g, provider);
         } else {
-            serializeFields(bean, gen, provider);
-        }
-        if (typeStr == null) {
-            typeSer.writeTypeSuffixForObject(bean, gen);
-        } else {
-            typeSer.writeCustomTypeSuffixForObject(bean, gen, typeStr);
+            serializeFields(bean, g, provider);
         }
+        typeSer.writeTypeSuffix(g, typeIdDef);
     }
-    
+
+    /**
+     * @since 2.9
+     */
+    protected final WritableTypeId _typeIdDef(TypeSerializer typeSer,
+            Object bean, JsonToken valueShape) {
+        if (_typeId == null) {
+            return typeSer.typeId(bean, valueShape);
+        }
+        Object typeId = _typeId.getValue(bean);
+        if (typeId == null) {
+            // 28-Jun-2017, tatu: Is this really needed? Unchanged from 2.8 but...
+            typeId = "";
+        }
+        return typeSer.typeId(bean, valueShape, typeId);
+    }
+
+    @Deprecated // since 2.9
     protected final String _customTypeId(Object bean)
     {
         final Object typeId = _typeId.getValue(bean);
@@ -666,7 +695,7 @@ public abstract class BeanSerializerBase
         }
         return (typeId instanceof String) ? (String) typeId : typeId.toString();
     }
-    
+
     /*
     /**********************************************************
     /* Field serialization methods
@@ -697,10 +726,9 @@ public abstract class BeanSerializerBase
             String name = (i == props.length) ? "[anySetter]" : props[i].getName();
             wrapAndThrow(provider, e, bean, name);
         } catch (StackOverflowError e) {
-            /* 04-Sep-2009, tatu: Dealing with this is tricky, since we do not
-             *   have many stack frames to spare... just one or two; can't
-             *   make many calls.
-             */
+            // 04-Sep-2009, tatu: Dealing with this is tricky, since we don't have many
+            //   stack frames to spare... just one or two; can't make many calls.
+
             // 10-Dec-2015, tatu: and due to above, avoid "from" method, call ctor directly:
             //JsonMappingException mapE = JsonMappingException.from(gen, "Infinite recursion (StackOverflowError)", e);
             JsonMappingException mapE = new JsonMappingException(gen, "Infinite recursion (StackOverflowError)", e);
