@@ -5,6 +5,7 @@ import java.lang.reflect.Type;
 import java.util.HashMap;
 
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.type.WritableTypeId;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JacksonStdImpl;
 import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatVisitorWrapper;
@@ -58,23 +59,24 @@ public class StdArraySerializers
     protected abstract static class TypedPrimitiveArraySerializer<T>
         extends ArraySerializerBase<T>
     {
-        /**
-         * Type serializer to use for values, if any.
-         */
-        protected final TypeSerializer _valueTypeSerializer;
-        
         protected TypedPrimitiveArraySerializer(Class<T> cls) {
             super(cls);
-            _valueTypeSerializer = null;
         }
 
         protected TypedPrimitiveArraySerializer(TypedPrimitiveArraySerializer<T> src,
-                BeanProperty prop, TypeSerializer vts, Boolean unwrapSingle) {
+                BeanProperty prop, Boolean unwrapSingle) {
             super(src, prop, unwrapSingle);
-            _valueTypeSerializer = vts;
+        }
+
+        // 01-Dec-2016, tatu: Only now realized that due strong typing of Java arrays,
+        //    we cannot really ever have value type serializers
+        @Override
+        public final ContainerSerializer<?> _withValueTypeSerializer(TypeSerializer vts) {
+            // throw exception or just do nothing?
+            return this;
         }
     }
-    
+
     /*
     /****************************************************************
     /* Concrete serializers, arrays
@@ -123,7 +125,7 @@ public class StdArraySerializers
         
         @Override
         public boolean isEmpty(SerializerProvider prov, boolean[] value) {
-            return (value == null) || (value.length == 0);
+            return value.length == 0;
         }
 
         @Override
@@ -135,22 +137,19 @@ public class StdArraySerializers
         public final void serialize(boolean[] value, JsonGenerator g, SerializerProvider provider) throws IOException
         {
             final int len = value.length;
-            if (len == 1) {
-                if (((_unwrapSingle == null) &&
-                        provider.isEnabled(SerializationFeature.WRITE_SINGLE_ELEM_ARRAYS_UNWRAPPED))
-                        || (_unwrapSingle == Boolean.TRUE)) {
-                    serializeContents(value, g, provider);
-                    return;
-                }
+            if ((len == 1) && _shouldUnwrapSingle(provider)) {
+                serializeContents(value, g, provider);
+                return;
             }
             g.writeStartArray(len);
+            g.setCurrentValue(value);
             serializeContents(value, g, provider);
             g.writeEndArray();
         }
-        
+
         @Override
         public void serializeContents(boolean[] value, JsonGenerator g, SerializerProvider provider)
-            throws IOException, JsonGenerationException
+            throws IOException
         {
             for (int i = 0, len = value.length; i < len; ++i) {
                 g.writeBoolean(value[i]);
@@ -182,18 +181,13 @@ public class StdArraySerializers
 
         public ShortArraySerializer() { super(short[].class); }
         public ShortArraySerializer(ShortArraySerializer src, BeanProperty prop,
-                TypeSerializer vts, Boolean unwrapSingle) {
-            super(src, prop, vts, unwrapSingle);
+                 Boolean unwrapSingle) {
+            super(src, prop, unwrapSingle);
         }
 
         @Override
         public JsonSerializer<?> _withResolved(BeanProperty prop,Boolean unwrapSingle) {
-            return new ShortArraySerializer(this, prop, _valueTypeSerializer, unwrapSingle);
-        }
-
-        @Override
-        public ContainerSerializer<?> _withValueTypeSerializer(TypeSerializer vts) {
-            return new ShortArraySerializer(this, _property, vts, _unwrapSingle);
+            return new ShortArraySerializer(this, prop, unwrapSingle);
         }
 
         @Override
@@ -209,7 +203,7 @@ public class StdArraySerializers
         
         @Override
         public boolean isEmpty(SerializerProvider prov, short[] value) {
-            return (value == null) || (value.length == 0);
+            return value.length == 0;
         }
 
         @Override
@@ -220,16 +214,13 @@ public class StdArraySerializers
         @Override
         public final void serialize(short[] value, JsonGenerator g, SerializerProvider provider) throws IOException
         {
-        	final int len = value.length;
-            if (len == 1) {
-                if (((_unwrapSingle == null) &&
-                        provider.isEnabled(SerializationFeature.WRITE_SINGLE_ELEM_ARRAYS_UNWRAPPED))
-                        || (_unwrapSingle == Boolean.TRUE)) {
-                    serializeContents(value, g, provider);
-                    return;
-                }
+            final int len = value.length;
+            if ((len == 1) && _shouldUnwrapSingle(provider)) {
+                serializeContents(value, g, provider);
+                return;
             }
             g.writeStartArray(len);
+            g.setCurrentValue(value);
             serializeContents(value, g, provider);
             g.writeEndArray();
         }
@@ -237,16 +228,8 @@ public class StdArraySerializers
         @SuppressWarnings("cast")
         @Override
         public void serializeContents(short[] value, JsonGenerator g, SerializerProvider provider)
-            throws IOException, JsonGenerationException
+            throws IOException
         {
-            if (_valueTypeSerializer != null) {
-                for (int i = 0, len = value.length; i < len; ++i) {
-                    _valueTypeSerializer.writeTypePrefixForScalar(null, g, Short.TYPE);
-                    g.writeNumber(value[i]);
-                    _valueTypeSerializer.writeTypeSuffixForScalar(null, g);
-                }
-                return;
-            }
             for (int i = 0, len = value.length; i < len; ++i) {
                 g.writeNumber((int)value[i]);
             }
@@ -273,7 +256,7 @@ public class StdArraySerializers
      * they are most likely to be textual data, and should be written as
      * Strings, not arrays of entries.
      *<p>
-     * NOTE: since it is NOT serialized as an array, can not use AsArraySerializer as base
+     * NOTE: since it is NOT serialized as an array, cannot use AsArraySerializer as base
      */
     @JacksonStdImpl
     public static class CharArraySerializer extends StdSerializer<char[]>
@@ -282,16 +265,17 @@ public class StdArraySerializers
         
         @Override
         public boolean isEmpty(SerializerProvider prov, char[] value) {
-            return (value == null) || (value.length == 0);
+            return value.length == 0;
         }
         
         @Override
         public void serialize(char[] value, JsonGenerator g, SerializerProvider provider)
-            throws IOException, JsonGenerationException
+            throws IOException
         {
             // [JACKSON-289] allows serializing as 'sparse' char array too:
             if (provider.isEnabled(SerializationFeature.WRITE_CHAR_ARRAYS_AS_JSON_ARRAYS)) {
                 g.writeStartArray(value.length);
+                g.setCurrentValue(value);
                 _writeArrayContents(g, value);
                 g.writeEndArray();
             } else {
@@ -302,22 +286,25 @@ public class StdArraySerializers
         @Override
         public void serializeWithType(char[] value, JsonGenerator g, SerializerProvider provider,
                 TypeSerializer typeSer)
-            throws IOException, JsonGenerationException
+            throws IOException
         {
             // [JACKSON-289] allows serializing as 'sparse' char array too:
-            if (provider.isEnabled(SerializationFeature.WRITE_CHAR_ARRAYS_AS_JSON_ARRAYS)) {
-                typeSer.writeTypePrefixForArray(value, g);
+            final boolean asArray = provider.isEnabled(SerializationFeature.WRITE_CHAR_ARRAYS_AS_JSON_ARRAYS);
+            WritableTypeId typeIdDef;
+            if (asArray) {
+                typeIdDef = typeSer.writeTypePrefix(g,
+                        typeSer.typeId(value, JsonToken.START_ARRAY));
                 _writeArrayContents(g, value);
-                typeSer.writeTypeSuffixForArray(value, g);
             } else { // default is to write as simple String
-                typeSer.writeTypePrefixForScalar(value, g);
+                typeIdDef = typeSer.writeTypePrefix(g,
+                        typeSer.typeId(value, JsonToken.VALUE_STRING));
                 g.writeString(value, 0, value.length);
-                typeSer.writeTypeSuffixForScalar(value, g);
             }
+            typeSer.writeTypeSuffix(g, typeIdDef);
         }
 
         private final void _writeArrayContents(JsonGenerator g, char[] value)
-            throws IOException, JsonGenerationException
+            throws IOException
         {
             for (int i = 0, len = value.length; i < len; ++i) {
                 g.writeString(value, i, 1);
@@ -385,7 +372,7 @@ public class StdArraySerializers
         
         @Override
         public boolean isEmpty(SerializerProvider prov, int[] value) {
-            return (value == null) || (value.length == 0);
+            return value.length == 0;
         }
 
         @Override
@@ -397,13 +384,9 @@ public class StdArraySerializers
         public final void serialize(int[] value, JsonGenerator g, SerializerProvider provider) throws IOException
         {
             final int len = value.length;
-            if (len == 1) {
-                if (((_unwrapSingle == null) &&
-                        provider.isEnabled(SerializationFeature.WRITE_SINGLE_ELEM_ARRAYS_UNWRAPPED))
-                        || (_unwrapSingle == Boolean.TRUE)) {
-                    serializeContents(value, g, provider);
-                    return;
-                }
+            if ((len == 1) && _shouldUnwrapSingle(provider)) {
+                serializeContents(value, g, provider);
+                return;
             }
             // 11-May-2016, tatu: As per [core#277] we have efficient `writeArray(...)` available
             g.setCurrentValue(value);
@@ -440,18 +423,13 @@ public class StdArraySerializers
 
         public LongArraySerializer() { super(long[].class); }
         public LongArraySerializer(LongArraySerializer src, BeanProperty prop,
-                TypeSerializer vts, Boolean unwrapSingle) {
-            super(src, prop, vts, unwrapSingle);
+                Boolean unwrapSingle) {
+            super(src, prop, unwrapSingle);
         }
 
         @Override
         public JsonSerializer<?> _withResolved(BeanProperty prop,Boolean unwrapSingle) {
-            return new LongArraySerializer(this, prop, _valueTypeSerializer, unwrapSingle);
-        }
-
-        @Override
-        public ContainerSerializer<?> _withValueTypeSerializer(TypeSerializer vts) {
-            return new LongArraySerializer(this, _property, vts, _unwrapSingle);
+            return new LongArraySerializer(this, prop, unwrapSingle);
         }
 
         @Override
@@ -467,7 +445,7 @@ public class StdArraySerializers
         
         @Override
         public boolean isEmpty(SerializerProvider prov, long[] value) {
-            return (value == null) || (value.length == 0);
+            return value.length == 0;
         }
 
         @Override
@@ -479,13 +457,9 @@ public class StdArraySerializers
         public final void serialize(long[] value, JsonGenerator g, SerializerProvider provider) throws IOException
         {
             final int len = value.length;
-            if (len == 1) {
-                if (((_unwrapSingle == null) &&
-                        provider.isEnabled(SerializationFeature.WRITE_SINGLE_ELEM_ARRAYS_UNWRAPPED))
-                        || (_unwrapSingle == Boolean.TRUE)) {
-                    serializeContents(value, g, provider);
-                    return;
-                }
+            if ((len == 1) && _shouldUnwrapSingle(provider)) {
+                serializeContents(value, g, provider);
+                return;
             }
             // 11-May-2016, tatu: As per [core#277] we have efficient `writeArray(...)` available
             g.setCurrentValue(value);
@@ -496,15 +470,6 @@ public class StdArraySerializers
         public void serializeContents(long[] value, JsonGenerator g, SerializerProvider provider)
             throws IOException
         {
-            if (_valueTypeSerializer != null) {
-                for (int i = 0, len = value.length; i < len; ++i) {
-                    _valueTypeSerializer.writeTypePrefixForScalar(null, g, Long.TYPE);
-                    g.writeNumber(value[i]);
-                    _valueTypeSerializer.writeTypeSuffixForScalar(null, g);
-                }
-                return;
-            }
-            
             for (int i = 0, len = value.length; i < len; ++i) {
                 g.writeNumber(value[i]);
             }
@@ -536,18 +501,13 @@ public class StdArraySerializers
             super(float[].class);
         }
         public FloatArraySerializer(FloatArraySerializer src, BeanProperty prop,
-                TypeSerializer vts, Boolean unwrapSingle) {
-            super(src, prop, vts, unwrapSingle);
-        }
-
-        @Override
-        public ContainerSerializer<?> _withValueTypeSerializer(TypeSerializer vts) {
-            return new FloatArraySerializer(this, _property, vts, _unwrapSingle);
+                Boolean unwrapSingle) {
+            super(src, prop, unwrapSingle);
         }
 
         @Override
         public JsonSerializer<?> _withResolved(BeanProperty prop,Boolean unwrapSingle) {
-            return new FloatArraySerializer(this, prop, _valueTypeSerializer, unwrapSingle);
+            return new FloatArraySerializer(this, prop, unwrapSingle);
         }
 
         @Override
@@ -563,7 +523,7 @@ public class StdArraySerializers
         
         @Override
         public boolean isEmpty(SerializerProvider prov, float[] value) {
-            return (value == null) || (value.length == 0);
+            return value.length == 0;
         }
 
         @Override
@@ -575,31 +535,20 @@ public class StdArraySerializers
         public final void serialize(float[] value, JsonGenerator g, SerializerProvider provider) throws IOException
         {
             final int len = value.length;
-            if (len == 1) {
-                if (((_unwrapSingle == null) &&
-                        provider.isEnabled(SerializationFeature.WRITE_SINGLE_ELEM_ARRAYS_UNWRAPPED))
-                        || (_unwrapSingle == Boolean.TRUE)) {
-                    serializeContents(value, g, provider);
-                    return;
-                }
+            if ((len == 1) && _shouldUnwrapSingle(provider)) {
+                serializeContents(value, g, provider);
+                return;
             }
             g.writeStartArray(len);
+            g.setCurrentValue(value);
             serializeContents(value, g, provider);
             g.writeEndArray();
         }
         
         @Override
         public void serializeContents(float[] value, JsonGenerator g, SerializerProvider provider)
-            throws IOException, JsonGenerationException
+            throws IOException
         {
-            if (_valueTypeSerializer != null) {
-                for (int i = 0, len = value.length; i < len; ++i) {
-                    _valueTypeSerializer.writeTypePrefixForScalar(null, g, Float.TYPE);
-                    g.writeNumber(value[i]);
-                    _valueTypeSerializer.writeTypeSuffixForScalar(null, g);
-                }
-                return;
-            }
             for (int i = 0, len = value.length; i < len; ++i) {
                 g.writeNumber(value[i]);
             }
@@ -661,7 +610,7 @@ public class StdArraySerializers
         
         @Override
         public boolean isEmpty(SerializerProvider prov, double[] value) {
-            return (value == null) || (value.length == 0);
+            return value.length == 0;
         }
 
         @Override
@@ -673,16 +622,12 @@ public class StdArraySerializers
         public final void serialize(double[] value, JsonGenerator g, SerializerProvider provider) throws IOException
         {
             final int len = value.length;
-            if (len == 1) {
-                if (((_unwrapSingle == null) &&
-                        provider.isEnabled(SerializationFeature.WRITE_SINGLE_ELEM_ARRAYS_UNWRAPPED))
-                        || (_unwrapSingle == Boolean.TRUE)) {
-                    serializeContents(value, g, provider);
-                    return;
-                }
+            if ((len == 1) && _shouldUnwrapSingle(provider)) {
+                serializeContents(value, g, provider);
+                return;
             }
-            // 11-May-2016, tatu: As per [core#277] we have efficient `writeArray(...)` available
             g.setCurrentValue(value);
+            // 11-May-2016, tatu: As per [core#277] we have efficient `writeArray(...)` available
             g.writeArray(value, 0, value.length);
         }
 
