@@ -297,15 +297,13 @@ public final class TypeFactory
                 prob = ClassUtil.getRootCause(e);
             }
         }
-        if (prob instanceof RuntimeException) {
-            throw (RuntimeException) prob;
-        }
+        ClassUtil.throwIfRTE(prob);
         throw new ClassNotFoundException(prob.getMessage(), prob);
     }
     
     protected Class<?> classForName(String name, boolean initialize,
-                                   ClassLoader loader) throws ClassNotFoundException {
-    	return Class.forName(name, true, loader);
+            ClassLoader loader) throws ClassNotFoundException {
+        return Class.forName(name, true, loader);
     }
     
     protected Class<?> classForName(String name) throws ClassNotFoundException {
@@ -352,7 +350,7 @@ public final class TypeFactory
         // also: if we start from untyped, not much to save
         do { // bogus loop to be able to break
             if (rawBase == Object.class) {
-                newType = _fromClass(null, subclass, TypeBindings.emptyBindings());
+                newType = _fromClass(null, subclass, EMPTY_BINDINGS);
                 break;
             }
             if (!rawBase.isAssignableFrom(subclass)) {
@@ -363,7 +361,7 @@ public final class TypeFactory
 
             // (1) Original target type has no generics -- just resolve subtype
             if (baseType.getBindings().isEmpty()) {
-                newType = _fromClass(null, subclass, TypeBindings.emptyBindings());     
+                newType = _fromClass(null, subclass, EMPTY_BINDINGS);
                 break;
             }
             // (2) A small set of "well-known" List/Map subtypes where can take a short-cut
@@ -396,105 +394,120 @@ public final class TypeFactory
             // (3) Sub-class does not take type parameters -- just resolve subtype
             int typeParamCount = subclass.getTypeParameters().length;
             if (typeParamCount == 0) {
-                newType = _fromClass(null, subclass, TypeBindings.emptyBindings());     
+                newType = _fromClass(null, subclass, EMPTY_BINDINGS);
                 break;
             }
-            
-            // If not, we'll need to do more thorough forward+backwards resolution. Sigh.
-
-            // 20-Oct-2015, tatu: Container, Map-types somewhat special. There is
-            //    a way to fully resolve and merge hierarchies; but that gets expensive
-            //    so let's, for now, try to create close-enough approximation that
-            //    is not 100% same, structurally, but has equivalent information for
-            //    our specific neeeds.
-            // 29-Mar-2016, tatu: See [databind#1173]  (and test `TypeResolverTest`)
-            //  for a case where this code does get invoked: not ideal
-            // 29-Jun-2016, tatu: As to bindings, this works for [databind#1215], but
-            //  not certain it would reliably work... but let's hope for best for now
+            // (4) If all else fails, do the full traversal using placeholders
             TypeBindings tb = _bindingsForSubtype(baseType, typeParamCount, subclass);
-            if (baseType.isInterface()) {
-                newType = baseType.refine(subclass, tb, null, new JavaType[] { baseType });
-            } else {
-                newType = baseType.refine(subclass, tb, baseType, NO_TYPES);
-            }
-            // Only SimpleType returns null, but if so just resolve regularly
-            if (newType == null) {
-                newType = _fromClass(null, subclass, tb);
-            }
+            newType = _fromClass(null, subclass, tb);
+
         } while (false);
 
         // 25-Sep-2016, tatu: As per [databind#1384] also need to ensure handlers get
         //   copied as well
         newType = newType.withHandlersFrom(baseType);
         return newType;
-
-        // 20-Oct-2015, tatu: Old simplistic approach
-        
-        /*
-        // Currently mostly SimpleType instances can become something else
-        if (baseType instanceof SimpleType) {
-            // and only if subclass is an array, Collection or Map
-            if (subclass.isArray()
-                || Map.class.isAssignableFrom(subclass)
-                || Collection.class.isAssignableFrom(subclass)) {
-                // need to assert type compatibility...
-                if (!baseType.getRawClass().isAssignableFrom(subclass)) {
-                    throw new IllegalArgumentException("Class "+subclass.getClass().getName()+" not subtype of "+baseType);
-                }
-                // this _should_ work, right?
-                JavaType subtype = _fromClass(null, subclass, TypeBindings.emptyBindings());
-                // one more thing: handlers to copy?
-                Object h = baseType.getValueHandler();
-                if (h != null) {
-                    subtype = subtype.withValueHandler(h);
-                }
-                h = baseType.getTypeHandler();
-                if (h != null) {
-                    subtype = subtype.withTypeHandler(h);
-                }
-                return subtype;
-            }
-        }
-        // But there is the need for special case for arrays too, it seems
-        if (baseType instanceof ArrayType) {
-            if (subclass.isArray()) {
-                // actually see if it might be a no-op first:
-                ArrayType at = (ArrayType) baseType;
-                Class<?> rawComp = subclass.getComponentType();
-                if (at.getContentType().getRawClass() == rawComp) {
-                    return baseType;
-                }
-                JavaType componentType = _fromAny(null, rawComp, null);
-                return ((ArrayType) baseType).withComponentType(componentType);
-            }
-        }
-
-        // otherwise regular narrowing should work just fine
-        return baseType.narrowBy(subclass);
-        */
     }
 
     private TypeBindings _bindingsForSubtype(JavaType baseType, int typeParamCount, Class<?> subclass)
     {
-        // But otherwise gets bit tricky, as we need to partially resolve the type hierarchy
-        // (hopefully passing null Class for root is ok)
-        int baseCount = baseType.containedTypeCount();
-        if (baseCount == typeParamCount) {
-            if (typeParamCount == 1) {
-                return TypeBindings.create(subclass, baseType.containedType(0));
-            }
-            if (typeParamCount == 2) {
-                return TypeBindings.create(subclass, baseType.containedType(0),
-                        baseType.containedType(1));
-            }
-            List<JavaType> types = new ArrayList<JavaType>(baseCount);
-            for (int i = 0; i < baseCount; ++i) {
-                types.add(baseType.containedType(i));
-            }
-            return TypeBindings.create(subclass, types);
+        PlaceholderForType[] placeholders = new PlaceholderForType[typeParamCount];
+        for (int i = 0; i < typeParamCount; ++i) {
+            placeholders[i] = new PlaceholderForType(i);
         }
-        // Otherwise, two choices: match N first, or empty. Do latter, for now
-        return TypeBindings.emptyBindings();
+        TypeBindings b = TypeBindings.create(subclass, placeholders);
+        // First: pseudo-resolve to get placeholders in place:
+        JavaType tmpSub = _fromClass(null, subclass, b);
+        // Then find super-type
+        JavaType baseWithPlaceholders = tmpSub.findSuperType(baseType.getRawClass());
+        if (baseWithPlaceholders == null) { // should be found but...
+            throw new IllegalArgumentException(String.format(
+                    "Internal error: unable to locate supertype (%s) from resolved subtype %s", baseType.getRawClass().getName(),
+                    subclass.getName()));
+        }
+        // and traverse type hierarchies to both verify and to resolve placeholders
+        String error = _resolveTypePlaceholders(baseType, baseWithPlaceholders);
+        if (error != null) {
+            throw new IllegalArgumentException("Failed to specialize base type "+baseType.toCanonical()+" as "
+                    +subclass.getName()+", problem: "+error);
+        }
+
+        final JavaType[] typeParams = new JavaType[typeParamCount];
+        for (int i = 0; i < typeParamCount; ++i) {
+            JavaType t = placeholders[i].actualType();
+            // 18-Oct-2017, tatu: Looks like sometimes we have incomplete bindings (even if not
+            //     common, it is possible if subtype is type-erased class with added type
+            //     variable -- see test(s) with "bogus" type(s)).
+            if (t == null) {
+                t = unknownType();
+            }
+            typeParams[i] = t;
+        }
+        return TypeBindings.create(subclass, typeParams);
+    }
+
+    private String _resolveTypePlaceholders(JavaType sourceType, JavaType actualType)
+        throws IllegalArgumentException
+    {
+        List<JavaType> expectedTypes = sourceType.getBindings().getTypeParameters();
+        List<JavaType> actualTypes = actualType.getBindings().getTypeParameters();
+        for (int i = 0, len = expectedTypes.size(); i < len; ++i) {
+            JavaType exp = expectedTypes.get(i);
+            JavaType act = actualTypes.get(i);
+
+            if (!_verifyAndResolvePlaceholders(exp, act)) {
+                // 14-May-2018, tatu: As per [databind#2034] it seems we better relax assignment
+                //   rules further -- at least likely "raw" (untyped, non-generic) base should probably
+                //   allow specialization.
+                if (exp.hasRawClass(Object.class)) {
+                    continue;
+                }
+                // 19-Apr-2018, tatu: Hack for [databind#1964] -- allow type demotion
+                //    for `java.util.Map` key type if (and only if) target type is
+                //    `java.lang.Object`
+                if (i == 0) {
+                    if (sourceType.hasRawClass(Map.class)
+                            && act.hasRawClass(Object.class)) {
+                        continue;
+                    }
+                }
+                // 19-Nov-2018, tatu: To solve [databind#2155], let's allow type-compatible
+                //   assignment for interfaces at least...
+                if (exp.isInterface()) {
+                    if (exp.isTypeOrSuperTypeOf(act.getRawClass())) {
+                        continue;
+                    }
+                }
+                return String.format("Type parameter #%d/%d differs; can not specialize %s with %s",
+                        (i+1), len, exp.toCanonical(), act.toCanonical());
+            }
+        }
+        return null;
+    }
+
+    private boolean _verifyAndResolvePlaceholders(JavaType exp, JavaType act)
+    {
+        // See if we have an actual type placeholder to resolve; if yes, replace
+        if (act instanceof PlaceholderForType) {
+            ((PlaceholderForType) act).actualType(exp);
+            return true;
+        }
+        // if not, try to verify compatibility. But note that we can not
+        // use simple equality as we need to resolve recursively
+        if (exp.getRawClass() != act.getRawClass()) {
+            return false;
+        }
+        // But we can check type parameters "blindly"
+        List<JavaType> expectedTypes = exp.getBindings().getTypeParameters();
+        List<JavaType> actualTypes = act.getBindings().getTypeParameters();
+        for (int i = 0, len = expectedTypes.size(); i < len; ++i) {
+            JavaType exp2 = expectedTypes.get(i);
+            JavaType act2 = actualTypes.get(i);
+            if (!_verifyAndResolvePlaceholders(exp2, act2)) {
+                return false;
+            }
+        }
+        return true;
     }
 
     /**
@@ -662,7 +675,7 @@ public final class TypeFactory
     public JavaType constructType(Type type, JavaType contextType) {
         TypeBindings bindings;
         if (contextType == null) {
-            bindings = TypeBindings.emptyBindings();
+            bindings = EMPTY_BINDINGS;
         } else {
             bindings = contextType.getBindings();
             // 16-Nov-2016, tatu: Unfortunately as per [databind#1456] this can't
@@ -716,7 +729,8 @@ public final class TypeFactory
      * NOTE: type modifiers are NOT called on Collection type itself; but are called
      * for contained types.
      */
-    public CollectionType constructCollectionType(Class<? extends Collection> collectionClass, Class<?> elementClass) {
+    public CollectionType constructCollectionType(Class<? extends Collection> collectionClass,
+            Class<?> elementClass) {
         return constructCollectionType(collectionClass,
                 _fromClass(null, elementClass, EMPTY_BINDINGS));
     }
@@ -727,11 +741,23 @@ public final class TypeFactory
      * NOTE: type modifiers are NOT called on Collection type itself; but are called
      * for contained types.
      */
-    public CollectionType constructCollectionType(Class<? extends Collection> collectionClass, JavaType elementType) {
-        // 19-Oct-2015, tatu: Allow case of no-type-variables, since it seems likely to be
-        //    a valid use case here
-        return (CollectionType) _fromClass(null, collectionClass,
-                TypeBindings.create(collectionClass, elementType));
+    public CollectionType constructCollectionType(Class<? extends Collection> collectionClass,
+            JavaType elementType)
+    {
+        TypeBindings bindings = TypeBindings.createIfNeeded(collectionClass, elementType);
+        CollectionType result = (CollectionType) _fromClass(null, collectionClass, bindings);
+        // 17-May-2017, tatu: As per [databind#1415], we better verify bound values if (but only if)
+        //    type being resolved was non-generic (i.e.element type was ignored)
+        if (bindings.isEmpty() && (elementType != null)) {
+            JavaType t = result.findSuperType(Collection.class);
+            JavaType realET = t.getContentType();
+            if (!realET.equals(elementType)) {
+                throw new IllegalArgumentException(String.format(
+                        "Non-generic Collection class %s did not resolve to something with element type %s but %s ",
+                        ClassUtil.nameOf(collectionClass), elementType, realET));
+            }
+        }
+        return result;
     }
 
     /**
@@ -766,7 +792,8 @@ public final class TypeFactory
      * NOTE: type modifiers are NOT called on constructed type itself; but are called
      * for contained types.
      */
-    public MapType constructMapType(Class<? extends Map> mapClass, Class<?> keyClass, Class<?> valueClass) {
+    public MapType constructMapType(Class<? extends Map> mapClass,
+            Class<?> keyClass, Class<?> valueClass) {
         JavaType kt, vt;
         if (mapClass == Properties.class) {
             kt = vt = CORE_TYPE_STRING;
@@ -784,10 +811,26 @@ public final class TypeFactory
      * for contained types.
      */
     public MapType constructMapType(Class<? extends Map> mapClass, JavaType keyType, JavaType valueType) {
-        return (MapType) _fromClass(null, mapClass,
-                TypeBindings.create(mapClass, new JavaType[] {
-                        keyType, valueType
-                }));
+        TypeBindings bindings = TypeBindings.createIfNeeded(mapClass, new JavaType[] { keyType, valueType });
+        MapType result = (MapType) _fromClass(null, mapClass, bindings);
+        // 17-May-2017, tatu: As per [databind#1415], we better verify bound values if (but only if)
+        //    type being resolved was non-generic (i.e.element type was ignored)
+        if (bindings.isEmpty()) {
+            JavaType t = result.findSuperType(Map.class);
+            JavaType realKT = t.getKeyType();
+            if (!realKT.equals(keyType)) {
+                throw new IllegalArgumentException(String.format(
+                        "Non-generic Map class %s did not resolve to something with key type %s but %s ",
+                        ClassUtil.nameOf(mapClass), keyType, realKT));
+            }
+            JavaType realVT = t.getContentType();
+            if (!realVT.equals(valueType)) {
+                throw new IllegalArgumentException(String.format(
+                        "Non-generic Map class %s did not resolve to something with value type %s but %s ",
+                        ClassUtil.nameOf(mapClass), valueType, realVT));
+            }
+        }
+        return result;
     }
 
     /**
@@ -875,15 +918,15 @@ public final class TypeFactory
      * type <code>List&lt;Set&lt;Integer>></code>, you could
      * call
      *<pre>
-     *  JavaType inner = TypeFactory.constructParametrizedType(Set.class, Set.class, Integer.class);
-     *  return TypeFactory.constructParametrizedType(ArrayList.class, List.class, inner);
+     *  JavaType inner = TypeFactory.constructParametricType(Set.class, Set.class, Integer.class);
+     *  return TypeFactory.constructParametricType(ArrayList.class, List.class, inner);
      *</pre>
      *<p>
      * The reason for first two arguments to be separate is that parameterization may
      * apply to a super-type. For example, if generic type was instead to be
      * constructed for <code>ArrayList&lt;Integer></code>, the usual call would be:
      *<pre>
-     *  TypeFactory.constructParametrizedType(ArrayList.class, List.class, Integer.class);
+     *  TypeFactory.constructParametricType(ArrayList.class, List.class, Integer.class);
      *</pre>
      * since parameterization is applied to {@link java.util.List}.
      * In most cases distinction does not matter, but there are types where it does;
@@ -900,7 +943,7 @@ public final class TypeFactory
         int len = parameterClasses.length;
         JavaType[] pt = new JavaType[len];
         for (int i = 0; i < len; ++i) {
-            pt[i] = _fromClass(null, parameterClasses[i], null);
+            pt[i] = _fromClass(null, parameterClasses[i], EMPTY_BINDINGS);
         }
         return constructParametricType(parametrized, pt);
     }
@@ -911,15 +954,15 @@ public final class TypeFactory
      * type <code>List&lt;Set&lt;Integer>></code>, you could
      * call
      *<pre>
-     *  JavaType inner = TypeFactory.constructParametrizedType(Set.class, Set.class, Integer.class);
-     *  return TypeFactory.constructParametrizedType(ArrayList.class, List.class, inner);
+     *  JavaType inner = TypeFactory.constructParametricType(Set.class, Set.class, Integer.class);
+     *  return TypeFactory.constructParametricType(ArrayList.class, List.class, inner);
      *</pre>
      *<p>
      * The reason for first two arguments to be separate is that parameterization may
      * apply to a super-type. For example, if generic type was instead to be
      * constructed for <code>ArrayList&lt;Integer></code>, the usual call would be:
      *<pre>
-     *  TypeFactory.constructParametrizedType(ArrayList.class, List.class, Integer.class);
+     *  TypeFactory.constructParametricType(ArrayList.class, List.class, Integer.class);
      *</pre>
      * since parameterization is applied to {@link java.util.List}.
      * In most cases distinction does not matter, but there are types where it does;
@@ -939,7 +982,10 @@ public final class TypeFactory
 
     /**
      * @since 2.5 -- but will probably deprecated in 2.7 or 2.8 (not needed with 2.7)
+     *
+     * @deprecated since 2.9 Use {@link #constructParametricType(Class,JavaType...)} instead
      */
+    @Deprecated
     public JavaType constructParametrizedType(Class<?> parametrized, Class<?> parametersFor,
             JavaType... parameterTypes)
     {
@@ -948,7 +994,10 @@ public final class TypeFactory
 
     /**
      * @since 2.5 -- but will probably deprecated in 2.7 or 2.8 (not needed with 2.7)
+     *
+     * @deprecated since 2.9 Use {@link #constructParametricType(Class,Class...)} instead
      */
+    @Deprecated
     public JavaType constructParametrizedType(Class<?> parametrized, Class<?> parametersFor,
             Class<?>... parameterClasses)
     {
@@ -1048,7 +1097,7 @@ public final class TypeFactory
                 vt = typeParams.get(1);
                 break;
             default:
-                throw new IllegalArgumentException("Strange Map type "+rawClass.getName()+": can not determine type parameters");
+                throw new IllegalArgumentException("Strange Map type "+rawClass.getName()+": cannot determine type parameters");
             }
         }
         return MapType.construct(rawClass, bindings, superClass, superInterfaces, kt, vt);
@@ -1065,7 +1114,7 @@ public final class TypeFactory
         } else if (typeParams.size() == 1) {
             ct = typeParams.get(0);
         } else {
-            throw new IllegalArgumentException("Strange Collection type "+rawClass.getName()+": can not determine type parameters");
+            throw new IllegalArgumentException("Strange Collection type "+rawClass.getName()+": cannot determine type parameters");
         }
         return CollectionType.construct(rawClass, bindings, superClass, superInterfaces, ct);
     }
@@ -1081,7 +1130,7 @@ public final class TypeFactory
         } else if (typeParams.size() == 1) {
             ct = typeParams.get(0);
         } else {
-            throw new IllegalArgumentException("Strange Reference type "+rawClass.getName()+": can not determine type parameters");
+            throw new IllegalArgumentException("Strange Reference type "+rawClass.getName()+": cannot determine type parameters");
         }
         return ReferenceType.construct(rawClass, bindings, superClass, superInterfaces, ct);
     }
@@ -1260,7 +1309,7 @@ public final class TypeFactory
                 superClass = null;
                 superInterfaces = _resolveSuperInterfaces(context, rawType, bindings);
             } else {
-                // Note: even Enums can implement interfaces, so can not drop those
+                // Note: even Enums can implement interfaces, so cannot drop those
                 superClass = _resolveSuperClass(context, rawType, bindings);
                 superInterfaces = _resolveSuperInterfaces(context, rawType, bindings);
             }
@@ -1329,7 +1378,7 @@ public final class TypeFactory
             JavaType superClass, JavaType[] superInterfaces)
     {
         if (bindings == null) {
-            bindings = TypeBindings.emptyBindings();
+            bindings = EMPTY_BINDINGS;
         }
         
         // Quite simple when we resolving exact class/interface; start with that
@@ -1393,13 +1442,12 @@ public final class TypeFactory
         // always of type Class: if not, need to add more code to resolve it to Class.        
         Type[] args = ptype.getActualTypeArguments();
         int paramCount = (args == null) ? 0 : args.length;
-        JavaType[] pt;
         TypeBindings newBindings;        
 
         if (paramCount == 0) {
             newBindings = EMPTY_BINDINGS;
         } else {
-            pt = new JavaType[paramCount];
+            JavaType[] pt = new JavaType[paramCount];
             for (int i = 0; i < paramCount; ++i) {
                 pt[i] = _fromAny(context, args[i], parentBindings);
             }
@@ -1418,6 +1466,7 @@ public final class TypeFactory
     {
         // ideally should find it via bindings:
         final String name = var.getName();
+if (bindings == null) throw new Error("No Bindings!");
         JavaType type = bindings.findBoundType(name);
         if (type != null) {
             return type;
