@@ -1,12 +1,10 @@
 package com.fasterxml.jackson.databind.ext;
 
-import java.nio.file.FileSystem;
 import java.nio.file.Path;
+import java.nio.file.Paths;
 
 import com.fasterxml.jackson.databind.*;
-
-import com.google.common.jimfs.Configuration;
-import com.google.common.jimfs.Jimfs;
+import com.fasterxml.jackson.databind.ObjectMapper.DefaultTyping;
 
 /**
  * @since 2.7
@@ -17,17 +15,32 @@ public class TestJava7Types extends BaseMapTest
     {
         ObjectMapper mapper = new ObjectMapper();
 
-        FileSystem fs = Jimfs.newFileSystem(Configuration.unix());
-        Path input = fs.getPath("/tmp", "foo.txt");
+        Path input = Paths.get("/tmp", "foo.txt");
 
         String json = mapper.writeValueAsString(input);
         assertNotNull(json);
-        
+
         Path p = mapper.readValue(json, Path.class);
         assertNotNull(p);
         
         assertEquals(input.toUri(), p.toUri());
         assertEquals(input, p);
-        fs.close();
+    }
+
+    // [databind#1688]:
+    public void testPolymorphicPath() throws Exception
+    {
+        ObjectMapper mapper = new ObjectMapper();
+        mapper.enableDefaultTyping(DefaultTyping.NON_FINAL);
+        Path input = Paths.get("/tmp", "foo.txt");
+
+        String json = mapper.writeValueAsString(new Object[] { input });
+
+        Object[] obs = mapper.readValue(json, Object[].class);
+        assertEquals(1, obs.length);
+        Object ob = obs[0];
+        assertTrue(ob instanceof Path);
+
+        assertEquals(input.toString(), ob.toString());
     }
 }
