@@ -4,7 +4,6 @@ import java.io.*;
 import java.util.*;
 
 import com.fasterxml.jackson.annotation.*;
-import com.fasterxml.jackson.annotation.JsonFormat.Shape;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.*;
@@ -114,32 +113,6 @@ public class TestEnumSerialization
         protected String key;
         OK(String key) { this.key = key; }
     }
-    
-    // Types for [https://github.com/FasterXML/jackson-databind/issues/24]
-    // (Enums as JSON Objects)
-
-    @JsonFormat(shape=JsonFormat.Shape.OBJECT)
-    static enum PoNUM {
-        A("a1"), B("b2");
-
-        @JsonProperty
-        protected final String value;
-        
-        private PoNUM(String v) { value = v; }
-
-        public String getValue() { return value; }
-    }
-
-    static class PoNUMContainer {
-        @JsonFormat(shape=Shape.NUMBER)
-        public OK text = OK.V1;
-    }
-    
-    @JsonFormat(shape=JsonFormat.Shape.ARRAY) // alias for 'number', as of 2.5
-    static enum PoAsArray
-    {
-        A, B;
-    }
 
     @SuppressWarnings({ "rawtypes", "serial" })
     static class LowerCasingEnumSerializer extends StdSerializer<Enum>
@@ -152,19 +125,6 @@ public class TestEnumSerialization
         }
     }
 
-    // for [databind#572]
-    static class PoOverrideAsString
-    {
-        @JsonFormat(shape=Shape.STRING)
-        public PoNUM value = PoNUM.B;
-    }
-
-    static class PoOverrideAsNumber
-    {
-        @JsonFormat(shape=Shape.NUMBER)
-        public PoNUM value = PoNUM.B;
-    }
-
     static enum MyEnum594 {
         VALUE_WITH_A_REALLY_LONG_NAME_HERE("longValue");
 
@@ -321,11 +281,17 @@ public class TestEnumSerialization
         String json = MAPPER.writeValueAsString(bean);
         assertEquals("{\"map\":{\"B\":3}}", json);
 
-        ObjectMapper m2 = new ObjectMapper();
         // but can change
-        m2.enable(SerializationFeature.WRITE_ENUMS_USING_TO_STRING);
-        json = m2.writeValueAsString(bean);
+        json = MAPPER.writer()
+                .with(SerializationFeature.WRITE_ENUMS_USING_TO_STRING)
+                .writeValueAsString(bean);
         assertEquals("{\"map\":{\"b\":3}}", json);
+
+        // [databind#1570]
+        json = MAPPER.writer()
+                .with(SerializationFeature.WRITE_ENUMS_USING_INDEX)
+                .writeValueAsString(bean);
+        assertEquals(aposToQuotes("{'map':{'"+TestEnum.B.ordinal()+"':3}}"), json);
     }
 
     public void testAsIndex() throws Exception
@@ -348,22 +314,6 @@ public class TestEnumSerialization
         assertEquals(quote("V2"), MAPPER.writeValueAsString(NOT_OK2.V2));
     }
 
-    // Tests for [issue#24]
-
-    public void testEnumAsObjectValid() throws Exception {
-        assertEquals("{\"value\":\"a1\"}", MAPPER.writeValueAsString(PoNUM.A));
-    }
-
-    public void testEnumAsIndexViaAnnotations() throws Exception {
-        assertEquals("{\"text\":0}", MAPPER.writeValueAsString(new PoNUMContainer()));
-    }
-
-    // As of 2.5, use of Shape.ARRAY is legal alias for "write as number"
-    public void testEnumAsObjectBroken() throws Exception
-    {
-        assertEquals("0", MAPPER.writeValueAsString(PoAsArray.A));
-    }
-    
     // [Issue#227]
     public void testGenericEnumSerializer() throws Exception
     {
@@ -375,15 +325,6 @@ public class TestEnumSerialization
         assertEquals(quote("b"), m.writeValueAsString(TestEnum.B));
     }
 
-    // [databind#572]
-    public void testOverrideEnumAsString() throws Exception {
-        assertEquals("{\"value\":\"B\"}", MAPPER.writeValueAsString(new PoOverrideAsString()));
-    }
-
-    public void testOverrideEnumAsNumber() throws Exception {
-        assertEquals("{\"value\":1}", MAPPER.writeValueAsString(new PoOverrideAsNumber()));
-    }
-
     // [databind#594]
     public void testJsonValueForEnumMapKey() throws Exception {
         assertEquals(aposToQuotes("{'stuff':{'longValue':'foo'}}"),
@@ -412,7 +353,7 @@ public class TestEnumSerialization
         m.put(LC749Enum.A, "value");
         assertEquals("{\"A\":\"value\"}", w.writeValueAsString(m));
     }
-    
+
     public void testEnumMapSerEnableToString() throws Exception {
         final ObjectMapper mapper = new ObjectMapper();
         ObjectWriter w = mapper.writer().with(SerializationFeature.WRITE_ENUMS_USING_TO_STRING);
@@ -426,6 +367,13 @@ public class TestEnumSerialization
         assertEquals(quote("aleph"), MAPPER.writeValueAsString(EnumWithJsonProperty.A));
     }
 
+    // [databind#1535]
+    public void testEnumKeysWithJsonProperty() throws Exception {
+        Map<EnumWithJsonProperty,Integer> input = new HashMap<EnumWithJsonProperty,Integer>();
+        input.put(EnumWithJsonProperty.A, 13);
+        assertEquals(aposToQuotes("{'aleph':13}"), MAPPER.writeValueAsString(input));
+    }
+
     // [databind#1322]
     public void testEnumsWithJsonPropertyInSet() throws Exception
     {
