@@ -40,10 +40,11 @@ public class TestTypeBindings
     /**********************************************************
      */
 
+    private final TypeFactory DEFAULT_TF = TypeFactory.defaultInstance();
+
     public void testInnerType() throws Exception
     {
-        TypeFactory tf = TypeFactory.defaultInstance();
-        JavaType type = tf.constructType(InnerGenericTyping.InnerClass.class);
+        JavaType type = DEFAULT_TF.constructType(InnerGenericTyping.InnerClass.class);
         assertEquals(MapType.class, type.getClass());
         JavaType keyType = type.getKeyType();
         assertEquals(Object.class, keyType.getRawClass());
@@ -56,8 +57,34 @@ public class TestTypeBindings
     // for [databind#76]
     public void testRecursiveType()
     {
-        TypeFactory tf = TypeFactory.defaultInstance();
-        JavaType type = tf.constructType(HashTree.class);
+        JavaType type = DEFAULT_TF.constructType(HashTree.class);
         assertNotNull(type);
     }
+
+    public void testBindingsBasics()
+    {
+        TypeBindings b = TypeBindings.create(Collection.class,
+                TypeFactory.unknownType());
+        // let's just call it -- should probably try to inspect but...
+        assertNotNull(b.toString());
+        assertEquals(Object.class, b.getBoundType(0).getRawClass());
+        assertNull(b.getBoundName(-1));
+        assertNull(b.getBoundType(-1));
+        assertNull(b.getBoundName(1));
+        assertNull(b.getBoundType(1));
+
+        assertFalse(b.equals("foo"));
+    }
+    
+    public void testInvalidBindings()
+    {
+        JavaType unknown = TypeFactory.unknownType();
+        try {
+            TypeBindings.create(AbstractType.class, unknown);
+            fail("Should not pass");
+        } catch (IllegalArgumentException e) {
+            verifyException(e, "Cannot create TypeBindings");
+            verifyException(e, "class expects 2");
+        }
+    }
 }
