@@ -1,30 +1,61 @@
 package com.fasterxml.jackson.databind.util;
 
 import java.io.*;
+import java.math.BigDecimal;
+import java.math.BigInteger;
 import java.util.UUID;
 
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.JsonParser.NumberType;
+import com.fasterxml.jackson.core.io.SerializedString;
 import com.fasterxml.jackson.core.util.JsonParserSequence;
-import com.fasterxml.jackson.databind.BaseMapTest;
-import com.fasterxml.jackson.databind.ObjectMapper;
+
+import com.fasterxml.jackson.databind.*;
 
 public class TestTokenBuffer extends BaseMapTest
 {
     private final ObjectMapper MAPPER = objectMapper();
-    
+
+    static class Base1730 { }
+
+    static class Sub1730 extends Base1730 { }
+
     /*
     /**********************************************************
     /* Basic TokenBuffer tests
     /**********************************************************
      */
-    
+
+    public void testBasicConfig() throws IOException
+    {
+        TokenBuffer buf;
+
+        buf = new TokenBuffer(MAPPER, false);
+        assertEquals(MAPPER.version(), buf.version());
+        assertSame(MAPPER, buf.getCodec());
+        assertNotNull(buf.getOutputContext());
+        assertFalse(buf.isClosed());
+
+        buf.setCodec(null);
+        assertNull(buf.getCodec());
+
+        assertFalse(buf.isEnabled(JsonGenerator.Feature.ESCAPE_NON_ASCII));
+        buf.enable(JsonGenerator.Feature.ESCAPE_NON_ASCII);
+        assertTrue(buf.isEnabled(JsonGenerator.Feature.ESCAPE_NON_ASCII));
+        buf.disable(JsonGenerator.Feature.ESCAPE_NON_ASCII);
+        assertFalse(buf.isEnabled(JsonGenerator.Feature.ESCAPE_NON_ASCII));
+
+        buf.close();
+        assertTrue(buf.isClosed());
+    }
+
     /**
      * Test writing of individual simple values
      */
     public void testSimpleWrites() throws IOException
     {
         TokenBuffer buf = new TokenBuffer(null, false); // no ObjectCodec
-
+        
         // First, with empty buffer
         JsonParser p = buf.asParser();
         assertNull(p.getCurrentToken());
@@ -34,7 +65,6 @@ public class TestTokenBuffer extends BaseMapTest
         // Then with simple text
         buf.writeString("abc");
 
-        // First, simple text
         p = buf.asParser();
         assertNull(p.getCurrentToken());
         assertToken(JsonToken.VALUE_STRING, p.nextToken());
@@ -54,6 +84,100 @@ public class TestTokenBuffer extends BaseMapTest
         buf.close();
     }
 
+    // For 2.9, explicit "isNaN" check
+    public void testSimpleNumberWrites() throws IOException
+    {
+        TokenBuffer buf = new TokenBuffer(null, false);
+
+        double[] values1 = new double[] {
+                0.25, Double.NaN, -2.0, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY
+        };
+        float[] values2 = new float[] {
+                Float.NEGATIVE_INFINITY,
+                0.25f,
+                Float.POSITIVE_INFINITY
+        };
+
+        for (double v : values1) {
+            buf.writeNumber(v);
+        }
+        for (float v : values2) {
+            buf.writeNumber(v);
+        }
+
+        JsonParser p = buf.asParser();
+        assertNull(p.getCurrentToken());
+
+        for (double v : values1) {
+            assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+            double actual = p.getDoubleValue();
+            boolean expNan = Double.isNaN(v) || Double.isInfinite(v);
+            assertEquals(expNan, p.isNaN());
+            assertEquals(0, Double.compare(v, actual));
+        }
+        for (float v : values2) {
+            assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+            float actual = p.getFloatValue();
+            boolean expNan = Float.isNaN(v) || Float.isInfinite(v);
+            assertEquals(expNan, p.isNaN());
+            assertEquals(0, Float.compare(v, actual));
+        }
+        p.close();
+        buf.close();
+    }
+
+    // [databind#1729]
+    public void testNumberOverflowInt() throws IOException
+    {
+        try (TokenBuffer buf = new TokenBuffer(null, false)) {
+            long big = 1L + Integer.MAX_VALUE;
+            buf.writeNumber(big);
+            try (JsonParser p = buf.asParser()) {
+                assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
+                assertEquals(NumberType.LONG, p.getNumberType());
+                try {
+                    p.getIntValue();
+                    fail("Expected failure for `int` overflow");
+                } catch (JsonParseException e) {
+                    verifyException(e, "Numeric value ("+big+") out of range of int");
+                }
+            }
+        }
+        // and ditto for coercion.
+        try (TokenBuffer buf = new TokenBuffer(null, false)) {
+            long big = 1L + Integer.MAX_VALUE;
+            buf.writeNumber(String.valueOf(big));
+            try (JsonParser p = buf.asParser()) {
+                // NOTE: oddity of buffering, no inspection of "real" type if given String...
+                assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+                try {
+                    p.getIntValue();
+                    fail("Expected failure for `int` overflow");
+                } catch (JsonParseException e) {
+                    verifyException(e, "Numeric value ("+big+") out of range of int");
+                }
+            }
+        }
+    }
+
+    public void testNumberOverflowLong() throws IOException
+    {
+        try (TokenBuffer buf = new TokenBuffer(null, false)) {
+            BigInteger big = BigInteger.valueOf(Long.MAX_VALUE).add(BigInteger.ONE);
+            buf.writeNumber(big);
+            try (JsonParser p = buf.asParser()) {
+                assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
+                assertEquals(NumberType.BIG_INTEGER, p.getNumberType());
+                try {
+                    p.getLongValue();
+                    fail("Expected failure for `long` overflow");
+                } catch (JsonParseException e) {
+                    verifyException(e, "Numeric value ("+big+") out of range of long");
+                }
+            }
+        }
+    }
+
     public void testParentContext() throws IOException
     {
         TokenBuffer buf = new TokenBuffer(null, false); // no ObjectCodec
@@ -200,6 +324,11 @@ public class TestTokenBuffer extends BaseMapTest
         verifyJsonSpecSampleDoc(tb.asParser(), true);
         tb.close();
         p.close();
+
+    
+        // 19-Oct-2016, tatu: Just for fun, trigger `toString()` for code coverage
+        String desc = tb.toString();
+        assertNotNull(desc);
     }
 
     public void testAppend() throws IOException
@@ -382,6 +511,39 @@ public class TestTokenBuffer extends BaseMapTest
         buf.close();
     }
 
+    public void testBasicSerialize() throws IOException
+    {
+        TokenBuffer buf;
+
+        // let's see how empty works...
+        buf = new TokenBuffer(MAPPER, false);
+        assertEquals("", MAPPER.writeValueAsString(buf));
+        buf.close();
+        
+        buf = new TokenBuffer(MAPPER, false);
+        buf.writeStartArray();
+        buf.writeBoolean(true);
+        buf.writeBoolean(false);
+        long l = 1L + Integer.MAX_VALUE;
+        buf.writeNumber(l);
+        buf.writeNumber((short) 4);
+        buf.writeNumber(0.5);
+        buf.writeEndArray();
+        assertEquals(aposToQuotes("[true,false,"+l+",4,0.5]"), MAPPER.writeValueAsString(buf));
+        buf.close();
+
+        buf = new TokenBuffer(MAPPER, false);
+        buf.writeStartObject();
+        buf.writeFieldName(new SerializedString("foo"));
+        buf.writeNull();
+        buf.writeFieldName("bar");
+        buf.writeNumber(BigInteger.valueOf(123));
+        buf.writeFieldName("dec");
+        buf.writeNumber(BigDecimal.valueOf(5).movePointLeft(2));
+        assertEquals(aposToQuotes("{'foo':null,'bar':123,'dec':0.05}"), MAPPER.writeValueAsString(buf));
+        buf.close();
+    }
+
     /*
     /**********************************************************
     /* Tests to verify interaction of TokenBuffer and JsonParserSequence
@@ -468,7 +630,7 @@ public class TestTokenBuffer extends BaseMapTest
         buf2.close();
         buf3.close();
         buf4.close();
-    }    
+    }
 
     // [databind#743]
     public void testRawValues() throws Exception
@@ -487,4 +649,20 @@ public class TestTokenBuffer extends BaseMapTest
         // then verify it would be serialized just fine
         assertEquals(RAW, MAPPER.writeValueAsString(buf));
     }
+
+    // [databind#1730]
+    public void testEmbeddedObjectCoerceCheck() throws Exception
+    {
+        TokenBuffer buf = new TokenBuffer(null, false);
+        Object inputPojo = new Sub1730();
+        buf.writeEmbeddedObject(inputPojo);
+
+        // first: raw value won't be transformed in any way:
+        JsonParser p = buf.asParser();
+        Base1730 out = MAPPER.readValue(p, Base1730.class);
+
+        assertSame(inputPojo, out);
+        p.close();
+        buf.close();
+    }
 }
